@@ -17,9 +17,13 @@
 /*global emval_get_global*/
 
 var LibraryEmVal = {
-  $emval_handle_array: [{},
-    {value: undefined},{value: null},
-    {value: true},{value: false}], // reserve zero and special values
+  $emval_handle_array: [
+    {},
+    {value: undefined},
+    {value: null},
+    {value: true},
+    {value: false}
+  ], // reserve zero and special values
   $emval_free_list: [],
   $emval_symbols: {}, // address -> string
 
@@ -34,9 +38,9 @@ var LibraryEmVal = {
   $count_emval_handles: function() {
     var count = 0;
     for (var i = 5; i < emval_handle_array.length; ++i) {
-        if (emval_handle_array[i] !== undefined) {
-            ++count;
-        }
+      if (emval_handle_array[i] !== undefined) {
+        ++count;
+      }
     }
     return count;
   },
@@ -44,9 +48,9 @@ var LibraryEmVal = {
   $get_first_emval__deps: ['$emval_handle_array'],
   $get_first_emval: function() {
     for (var i = 5; i < emval_handle_array.length; ++i) {
-        if (emval_handle_array[i] !== undefined) {
-            return emval_handle_array[i];
-        }
+      if (emval_handle_array[i] !== undefined) {
+        return emval_handle_array[i];
+      }
     }
     return null;
   },
@@ -60,36 +64,35 @@ var LibraryEmVal = {
   $getStringOrSymbol: function(address) {
     var symbol = emval_symbols[address];
     if (symbol === undefined) {
-        return readLatin1String(address);
-    } else {
-        return symbol;
+      return readLatin1String(address);
     }
+    return symbol;
   },
 
   $Emval__deps: ['$emval_handle_array', '$emval_free_list', '$throwBindingError', '$init_emval'],
   $Emval: {
-    toValue: function(handle) {
+    toValue: (handle) => {
       if (!handle) {
           throwBindingError('Cannot use deleted val. handle = ' + handle);
       }
       return emval_handle_array[handle].value;
     },
 
-    toHandle: function(value) {
+    toHandle: (value) => {
       switch (value) {
-        case undefined :{ return 1; }
-        case null :{ return 2; }
-        case true :{ return 3; }
-        case false :{ return 4; }
+        case undefined: return 1;
+        case null: return 2;
+        case true: return 3;
+        case false: return 4;
         default:{
           var handle = emval_free_list.length ?
               emval_free_list.pop() :
               emval_handle_array.length;
-  
+
           emval_handle_array[handle] = {refcount: 1, value: value};
           return handle;
-          }
         }
+      }
     }
   },
 
@@ -97,7 +100,7 @@ var LibraryEmVal = {
   _emval_incref__deps: ['$emval_handle_array'],
   _emval_incref: function(handle) {
     if (handle > 4) {
-        emval_handle_array[handle].refcount += 1;
+      emval_handle_array[handle].refcount += 1;
     }
   },
 
@@ -105,8 +108,8 @@ var LibraryEmVal = {
   _emval_decref__deps: ['$emval_free_list', '$emval_handle_array'],
   _emval_decref: function(handle) {
     if (handle > 4 && 0 === --emval_handle_array[handle].refcount) {
-        emval_handle_array[handle] = undefined;
-        emval_free_list.push(handle);
+      emval_handle_array[handle] = undefined;
+      emval_free_list.push(handle);
     }
   },
 
@@ -134,6 +137,18 @@ var LibraryEmVal = {
     return Emval.toHandle(getStringOrSymbol(v));
   },
 
+  _emval_new_u8string__sig: 'ii',
+  _emval_new_u8string__deps: ['$Emval'],
+  _emval_new_u8string: function(v) {
+    return Emval.toHandle(UTF8ToString(v));
+  },
+
+  _emval_new_u16string__sig: 'ii',
+  _emval_new_u16string__deps: ['$Emval'],
+  _emval_new_u16string: function(v) {
+    return Emval.toHandle(UTF16ToString(v));
+  },
+
   _emval_take_value__sig: 'iii',
   _emval_take_value__deps: ['$Emval', '$requireRegisteredType'],
   _emval_take_value: function(type, argv) {
@@ -148,22 +163,22 @@ var LibraryEmVal = {
     /*This function returns a new function that looks like this:
     function emval_allocator_3(constructor, argTypes, args) {
         var argType0 = requireRegisteredType(HEAP32[(argTypes >> 2)], "parameter 0");
-        var arg0 = argType0.readValueFromPointer(args);
+        var arg0 = argType0['readValueFromPointer'](args);
         var argType1 = requireRegisteredType(HEAP32[(argTypes >> 2) + 1], "parameter 1");
-        var arg1 = argType1.readValueFromPointer(args + 8);
+        var arg1 = argType1['readValueFromPointer'](args + 8);
         var argType2 = requireRegisteredType(HEAP32[(argTypes >> 2) + 2], "parameter 2");
-        var arg2 = argType2.readValueFromPointer(args + 16);
+        var arg2 = argType2['readValueFromPointer'](args + 16);
         var obj = new constructor(arg0, arg1, arg2);
         return Emval.toHandle(obj);
     } */
-#if DYNAMIC_EXECUTION == 0
+#if !DYNAMIC_EXECUTION
     var argsList = new Array(argCount + 1);
     return function(constructor, argTypes, args) {
       argsList[0] = constructor;
       for (var i = 0; i < argCount; ++i) {
         var argType = requireRegisteredType(HEAP32[(argTypes >> 2) + i], 'parameter ' + i);
-        argsList[i + 1] = argType.readValueFromPointer(args);
-        args += argType.argPackAdvance;
+        argsList[i + 1] = argType['readValueFromPointer'](args);
+        args += argType['argPackAdvance'];
       }
       var obj = new (constructor.bind.apply(constructor, argsList));
       return Emval.toHandle(obj);
@@ -171,7 +186,7 @@ var LibraryEmVal = {
 #else
     var argsList = "";
     for (var i = 0; i < argCount; ++i) {
-        argsList += (i!==0?", ":"")+"arg"+i; // 'arg0, arg1, ..., argn'
+      argsList += (i!==0?", ":"")+"arg"+i; // 'arg0, arg1, ..., argn'
     }
 
     var functionBody =
@@ -200,35 +215,35 @@ var LibraryEmVal = {
 
     var newer = emval_newers[argCount];
     if (!newer) {
-        newer = craftEmvalAllocator(argCount);
-        emval_newers[argCount] = newer;
+      newer = craftEmvalAllocator(argCount);
+      emval_newers[argCount] = newer;
     }
 
     return newer(handle, argTypes, args);
   },
 
-#if DYNAMIC_EXECUTION == 0
+#if !DYNAMIC_EXECUTION
   $emval_get_global: function() {
-    if (typeof globalThis === 'object') {
+    if (typeof globalThis == 'object') {
       return globalThis;
     }
     function testGlobal(obj) {
       obj['$$$embind_global$$$'] = obj;
-      var success = typeof $$$embind_global$$$ === 'object' && obj['$$$embind_global$$$'] === obj;
+      var success = typeof $$$embind_global$$$ == 'object' && obj['$$$embind_global$$$'] == obj;
       if (!success) {
         delete obj['$$$embind_global$$$'];
       }
       return success;
     }
-    if (typeof $$$embind_global$$$ === 'object') {
+    if (typeof $$$embind_global$$$ == 'object') {
       return $$$embind_global$$$;
     }
-    if (typeof global === 'object' && testGlobal(global)) {
+    if (typeof global == 'object' && testGlobal(global)) {
       $$$embind_global$$$ = global;
-    } else if (typeof self === 'object' && testGlobal(self)) {
+    } else if (typeof self == 'object' && testGlobal(self)) {
       $$$embind_global$$$ = self; // This works for both "window" and "self" (Web Workers) global objects
     }
-    if (typeof $$$embind_global$$$ === 'object') {
+    if (typeof $$$embind_global$$$ == 'object') {
       return $$$embind_global$$$;
     }
     throw Error('unable to get global object.');
@@ -236,7 +251,7 @@ var LibraryEmVal = {
 #else
   // appease jshint (technically this code uses eval)
   $emval_get_global: function() {
-    if (typeof globalThis === 'object') {
+    if (typeof globalThis == 'object') {
       return globalThis;
     }
     return (function(){
@@ -277,7 +292,7 @@ var LibraryEmVal = {
     value = Emval.toValue(value);
     handle[key] = value;
   },
-    
+
   _emval_as__sig: 'iiii',
   _emval_as__deps: ['$Emval', '$requireRegisteredType'],
   _emval_as: function(handle, returnType, destructorsRef) {
@@ -345,9 +360,9 @@ var LibraryEmVal = {
 
     var args = new Array(argCount);
     for (var i = 0; i < argCount; ++i) {
-        var type = types[i];
-        args[i] = type['readValueFromPointer'](argv);
-        argv += type['argPackAdvance'];
+      var type = types[i];
+      args[i] = type['readValueFromPointer'](argv);
+      argv += type['argPackAdvance'];
     }
 
     var rv = handle.apply(undefined, args);
@@ -358,9 +373,8 @@ var LibraryEmVal = {
   _emval_lookupTypes: function(argCount, argTypes) {
     var a = new Array(argCount);
     for (var i = 0; i < argCount; ++i) {
-        a[i] = requireRegisteredType(
-            HEAP32[(argTypes >> 2) + i],
-            "parameter " + i);
+      a[i] = requireRegisteredType(HEAP32[(argTypes >> 2) + i],
+                                   "parameter " + i);
     }
     return a;
   },
@@ -395,13 +409,13 @@ var LibraryEmVal = {
       return returnId;
     }
 
-#if DYNAMIC_EXECUTION == 0
+#if !DYNAMIC_EXECUTION
     var argN = new Array(argCount - 1);
-    var invokerFunction = function(handle, name, destructors, args) {
+    var invokerFunction = (handle, name, destructors, args) => {
       var offset = 0;
       for (var i = 0; i < argCount - 1; ++i) {
-        argN[i] = types[i + 1].readValueFromPointer(args + offset);
-        offset += types[i + 1].argPackAdvance;
+        argN[i] = types[i + 1]['readValueFromPointer'](args + offset);
+        offset += types[i + 1]['argPackAdvance'];
       }
       var rv = handle[name].apply(handle, argN);
       for (var i = 0; i < argCount - 1; ++i) {
@@ -410,19 +424,18 @@ var LibraryEmVal = {
         }
       }
       if (!retType.isVoid) {
-        return retType.toWireType(destructors, rv);
+        return retType['toWireType'](destructors, rv);
       }
     };
 #else
-
     var params = ["retType"];
     var args = [retType];
 
     var argsList = ""; // 'arg0, arg1, arg2, ... , argN'
     for (var i = 0; i < argCount - 1; ++i) {
-        argsList += (i !== 0 ? ", " : "") + "arg" + i;
-        params.push("argType" + i);
-        args.push(types[1 + i]);
+      argsList += (i !== 0 ? ", " : "") + "arg" + i;
+      params.push("argType" + i);
+      args.push(types[1 + i]);
     }
 
     var functionName = makeLegalFunctionName("methodCaller_" + signatureName);
@@ -491,13 +504,13 @@ var LibraryEmVal = {
   _emval_is_number__deps: ['$Emval'],
   _emval_is_number: function(handle) {
     handle = Emval.toValue(handle);
-    return typeof handle === 'number';
+    return typeof handle == 'number';
   },
 
   _emval_is_string__deps: ['$Emval'],
   _emval_is_string: function(handle) {
     handle = Emval.toValue(handle);
-    return typeof handle === 'string';
+    return typeof handle == 'string';
   },
 
   _emval_in__deps: ['$Emval'],
@@ -523,7 +536,7 @@ var LibraryEmVal = {
 #if ASYNCIFY
   _emval_await__deps: ['$Emval', '$Asyncify'],
   _emval_await: function(promise) {
-    return Asyncify.handleAsync(function () {
+    return Asyncify.handleAsync(() => {
       promise = Emval.toValue(promise);
       return promise.then(Emval.toHandle);
     });
