@@ -80,8 +80,9 @@ var LibraryGLFW = {
     },
 
   $GLFW__deps: ['emscripten_get_now', '$GL', '$Browser', '$GLFW_Window',
+    '$callUserCallback',
 #if FILESYSTEM
-    , '$FS'
+    '$FS',
 #endif
   ],
   $GLFW: {
@@ -191,7 +192,7 @@ var LibraryGLFW = {
         case 0xDB:return 91; // DOM_VK_OPEN_BRACKET -> GLFW_KEY_LEFT_BRACKET
         case 0xDC:return 92; // DOM_VK_BACKSLASH -> GLFW_KEY_BACKSLASH
         case 0xDD:return 93; // DOM_VK_CLOSE_BRACKET -> GLFW_KEY_RIGHT_BRACKET
-        case 0xC0:return 94; // DOM_VK_BACK_QUOTE -> GLFW_KEY_GRAVE_ACCENT
+        case 0xC0:return 96; // DOM_VK_BACK_QUOTE -> GLFW_KEY_GRAVE_ACCENT
 
 #if USE_GLFW == 2
         //#define GLFW_KEY_SPECIAL      256
@@ -528,13 +529,11 @@ var LibraryGLFW = {
 
 #if USE_GLFW == 3
       var sx = 0;
-      var sy = 0;
+      var sy = delta;
       if (event.type == 'mousewheel') {
         sx = event.wheelDeltaX;
-        sy = event.wheelDeltaY;
       } else {
         sx = event.deltaX;
-        sy = event.deltaY;
       }
 
       {{{ makeDynCall('vidd', 'GLFW.active.scrollFunc') }}}(GLFW.active.id, sx, sy);
@@ -591,13 +590,15 @@ var LibraryGLFW = {
 
       if (!GLFW.active.windowSizeFunc) return;
 
+      callUserCallback(function() {
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.width, GLFW.active.height);
+        {{{ makeDynCall('vii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.width, GLFW.active.height);
 #endif
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('viii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
+        {{{ makeDynCall('viii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
+      });
     },
 
     onFramebufferSizeChanged: function() {
@@ -605,9 +606,11 @@ var LibraryGLFW = {
 
       if (!GLFW.active.framebufferSizeFunc) return;
 
+      callUserCallback(function() {
 #if USE_GLFW == 3
-      {{{ makeDynCall('viii', 'GLFW.active.framebufferSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
+        {{{ makeDynCall('viii', 'GLFW.active.framebufferSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
+      });
     },
 
     getTime: function() {
@@ -632,13 +635,13 @@ var LibraryGLFW = {
     },
 
     joys: {}, // glfw joystick data
-    lastGamepadState: null,
+    lastGamepadState: [],
     lastGamepadStateFrame: null, // The integer value of Browser.mainLoop.currentFrameNumber of when the last gamepad state was produced.
 
     refreshJoysticks: function() {
       // Produce a new Gamepad API sample if we are ticking a new game frame, or if not using emscripten_set_main_loop() at all to drive animation.
       if (Browser.mainLoop.currentFrameNumber !== GLFW.lastGamepadStateFrame || !Browser.mainLoop.currentFrameNumber) {
-        GLFW.lastGamepadState = navigator.getGamepads ? navigator.getGamepads() : (navigator.webkitGetGamepads ? navigator.webkitGetGamepads : null);
+        GLFW.lastGamepadState = navigator.getGamepads ? navigator.getGamepads() : (navigator.webkitGetGamepads ? navigator.webkitGetGamepads : []);
         GLFW.lastGamepadStateFrame = Browser.mainLoop.currentFrameNumber;
 
         for (var joy = 0; joy < GLFW.lastGamepadState.length; ++joy) {
@@ -646,13 +649,13 @@ var LibraryGLFW = {
 
           if (gamepad) {
             if (!GLFW.joys[joy]) {
-              console.log('glfw joystick connected:',joy);
+              out('glfw joystick connected:',joy);
               GLFW.joys[joy] = {
-                id: allocate(intArrayFromString(gamepad.id), ALLOC_NORMAL),
+                id: allocateUTF8(gamepad.id),
                 buttonsCount: gamepad.buttons.length,
                 axesCount: gamepad.axes.length,
-                buttons: allocate(new Array(gamepad.buttons.length), ALLOC_NORMAL),
-                axes: allocate(new Array(gamepad.axes.length*4), ALLOC_NORMAL)
+                buttons: _malloc(gamepad.buttons.length),
+                axes: _malloc(gamepad.axes.length*4),
               };
 
               if (GLFW.joystickFunc) {
@@ -671,7 +674,7 @@ var LibraryGLFW = {
             }
           } else {
             if (GLFW.joys[joy]) {
-              console.log('glfw joystick disconnected',joy);
+              out('glfw joystick disconnected',joy);
 
               if (GLFW.joystickFunc) {
                 {{{ makeDynCall('vii', 'GLFW.joystickFunc') }}}(joy, 0x00040002); // GLFW_DISCONNECTED
@@ -743,7 +746,7 @@ var LibraryGLFW = {
       event.preventDefault();
 
 #if FILESYSTEM
-      var filenames = allocate(new Array(event.dataTransfer.files.length*4), ALLOC_NORMAL);
+      var filenames = _malloc(event.dataTransfer.files.length*4);
       var filenamesArray = [];
       var count = event.dataTransfer.files.length;
 
@@ -755,10 +758,10 @@ var LibraryGLFW = {
       function save(file) {
         var path = '/' + drop_dir + '/' + file.name.replace(/\//g, '_');
         var reader = new FileReader();
-        reader.onloadend = function(e) {
+        reader.onloadend = (e) => {
           if (reader.readyState != 2) { // not DONE
             ++written;
-            console.log('failed to read dropped file: '+file.name+': '+reader.error);
+            out('failed to read dropped file: '+file.name+': '+reader.error);
             return;
           }
 
@@ -775,7 +778,7 @@ var LibraryGLFW = {
         };
         reader.readAsArrayBuffer(file);
 
-        var filename = allocate(intArrayFromString(path), ALLOC_NORMAL);
+        var filename = allocateUTF8(path);
         filenamesArray.push(filename);
         setValue(filenames + i*4, filename, 'i8*');
       }
@@ -850,7 +853,7 @@ var LibraryGLFW = {
               break;
             }
             case 0x00034002: { // GLFW_CURSOR_HIDDEN
-              console.log("glfwSetInputMode called with GLFW_CURSOR_HIDDEN value not implemented.");
+              out("glfwSetInputMode called with GLFW_CURSOR_HIDDEN value not implemented.");
               break;
             }
             case 0x00034003: { // GLFW_CURSOR_DISABLED
@@ -860,22 +863,22 @@ var LibraryGLFW = {
               break;
             }
             default: {
-              console.log("glfwSetInputMode called with unknown value parameter value: " + value + ".");
+              out("glfwSetInputMode called with unknown value parameter value: " + value + ".");
               break;
             }
           }
           break;
         }
         case 0x00033002: { // GLFW_STICKY_KEYS
-          console.log("glfwSetInputMode called with GLFW_STICKY_KEYS mode not implemented.");
+          out("glfwSetInputMode called with GLFW_STICKY_KEYS mode not implemented.");
           break;
         }
         case 0x00033003: { // GLFW_STICKY_MOUSE_BUTTONS
-          console.log("glfwSetInputMode called with GLFW_STICKY_MOUSE_BUTTONS mode not implemented.");
+          out("glfwSetInputMode called with GLFW_STICKY_MOUSE_BUTTONS mode not implemented.");
           break;
         }
         default: {
-          console.log("glfwSetInputMode called with unknown mode parameter value: " + mode + ".");
+          out("glfwSetInputMode called with unknown mode parameter value: " + mode + ".");
           break;
         }
       }
@@ -1200,11 +1203,11 @@ var LibraryGLFW = {
       GLFW.extensions = UTF8ToString(_glGetString(0x1F03)).split(' ');
     }
 
-    if (GLFW.extensions.indexOf(extension) != -1) return 1;
+    if (GLFW.extensions.includes(extension)) return 1;
 
     // extensions from GLEmulations do not come unprefixed
     // so, try with prefix
-    return (GLFW.extensions.indexOf("GL_" + extension) != -1);
+    return (GLFW.extensions.includes("GL_" + extension));
   },
 
   glfwSwapInterval__deps: ['emscripten_set_main_loop_timing'],
@@ -1219,7 +1222,7 @@ var LibraryGLFW = {
   glfwGetVersionString__sig: 'i',
   glfwGetVersionString: function() {
     if (!GLFW.versionString) {
-      GLFW.versionString = allocate(intArrayFromString("3.2.1 JS WebGL Emscripten"), ALLOC_NORMAL);
+      GLFW.versionString = allocateUTF8("3.2.1 JS WebGL Emscripten");
     }
     return GLFW.versionString;
   },
@@ -1271,7 +1274,7 @@ var LibraryGLFW = {
   glfwGetMonitorName__sig: 'ii',
   glfwGetMonitorName: function(mon) {
     if (!GLFW.monitorString) {
-      GLFW.monitorString = allocate(intArrayFromString("HTML5 WebGL Canvas"), ALLOC_NORMAL);
+      GLFW.monitorString = allocateUTF8("HTML5 WebGL Canvas");
     }
     return GLFW.monitorString;
   },
@@ -1844,7 +1847,7 @@ var LibraryGLFW = {
   },
 
   glfwCreateThread: function(fun, arg) {
-    {{{ makeDynCall('vi', 'str') }}}(fun, arg);
+    {{{ makeDynCall('vi', 'fun') }}}(arg);
     // One single thread
     return 0;
   },
