@@ -5,10 +5,14 @@
  */
 
 var WasiLibrary = {
-  proc_exit__deps: ['exit'],
+  proc_exit__nothrow: true,
   proc_exit__sig: 'vi',
   proc_exit: function(code) {
-    _exit(code);
+#if MINIMAL_RUNTIME
+    throw 'exit(' + code + ')';
+#else
+    procExit(code);
+#endif
   },
 
   $getEnvStrings__deps: ['$ENV', '$getExecutableName'],
@@ -17,7 +21,7 @@ var WasiLibrary = {
       // Default values.
 #if !DETERMINISTIC
       // Browser language detection #8751
-      var lang = ((typeof navigator === 'object' && navigator.languages && navigator.languages[0]) || 'C').replace('-', '_') + '.UTF-8';
+      var lang = ((typeof navigator == 'object' && navigator.languages && navigator.languages[0]) || 'C').replace('-', '_') + '.UTF-8';
 #else
       // Deterministic language detection, ignore the browser's language.
       var lang = 'C.UTF-8';
@@ -49,6 +53,7 @@ var WasiLibrary = {
   },
 
   environ_sizes_get__deps: ['$getEnvStrings'],
+  environ_sizes_get__nothrow: true,
   environ_sizes_get__sig: 'iii',
   environ_sizes_get: function(penviron_count, penviron_buf_size) {
     var strings = getEnvStrings();
@@ -66,6 +71,7 @@ var WasiLibrary = {
     , '$writeAsciiToMemory'
 #endif
   ],
+  environ_get__nothrow: true,
   environ_get__sig: 'iii',
   environ_get: function(__environ, environ_buf) {
     var bufSize = 0;
@@ -81,31 +87,35 @@ var WasiLibrary = {
   // In normal (non-standalone) mode arguments are passed direclty
   // to main, and the `mainArgs` global does not exist.
 #if STANDALONE_WASM
+  args_sizes_get__nothrow: true,
   args_sizes_get__sig: 'iii',
   args_sizes_get: function(pargc, pargv_buf_size) {
+    {{{ from64(['pargc', 'pargv_buf_size']) }}};
 #if MAIN_READS_PARAMS
-    {{{ makeSetValue('pargc', 0, 'mainArgs.length', 'i32') }}};
+    {{{ makeSetValue('pargc', 0, 'mainArgs.length', SIZE_TYPE) }}};
     var bufSize = 0;
     mainArgs.forEach(function(arg) {
       bufSize += arg.length + 1;
     });
-    {{{ makeSetValue('pargv_buf_size', 0, 'bufSize', 'i32') }}};
+    {{{ makeSetValue('pargv_buf_size', 0, 'bufSize', SIZE_TYPE) }}};
 #else
-    {{{ makeSetValue('pargc', 0, '0', 'i32') }}};
+    {{{ makeSetValue('pargc', 0, '0', SIZE_TYPE) }}};
 #endif
     return 0;
   },
 
+  args_get__nothrow: true,
   args_get__sig: 'iii',
 #if MINIMAL_RUNTIME && MAIN_READS_PARAMS
   args_get__deps: ['$writeAsciiToMemory'],
 #endif
   args_get: function(argv, argv_buf) {
+    {{{ from64(['argv', 'argv_buf']) }}};
 #if MAIN_READS_PARAMS
     var bufSize = 0;
     mainArgs.forEach(function(arg, i) {
       var ptr = argv_buf + bufSize;
-      {{{ makeSetValue('argv', 'i * 4', 'ptr', 'i32') }}};
+      {{{ makeSetValue('argv', `i*${Runtime.POINTER_SIZE}`, 'ptr', POINTER_TYPE) }}};
       writeAsciiToMemory(arg, ptr);
       bufSize += arg.length + 1;
     });
@@ -125,8 +135,9 @@ var WasiLibrary = {
   // but the wasm file can't be legalized in standalone mode, which is where
   // this is needed. To get this code to be usable as a JS shim we need to
   // either wait for BigInt support or to legalize on the client.
+  clock_time_get__nothrow: true,
   clock_time_get__sig: 'iiiii',
-  clock_time_get__deps: ['emscripten_get_now', 'emscripten_get_now_is_monotonic', '$checkWasiClock'],
+  clock_time_get__deps: ['emscripten_get_now', '$nowIsMonotonic', '$checkWasiClock'],
   clock_time_get: function(clk_id, {{{ defineI64Param('precision') }}}, ptime) {
     {{{ receiveI64ParamAsI32s('precision') }}}
     if (!checkWasiClock(clk_id)) {
@@ -136,7 +147,7 @@ var WasiLibrary = {
     // all wasi clocks but realtime are monotonic
     if (clk_id === {{{ cDefine('__WASI_CLOCKID_REALTIME') }}}) {
       now = Date.now();
-    } else if (_emscripten_get_now_is_monotonic) {
+    } else if (nowIsMonotonic) {
       now = _emscripten_get_now();
     } else {
       return {{{ cDefine('ENOSYS') }}};
@@ -148,8 +159,9 @@ var WasiLibrary = {
     return 0;
   },
 
+  clock_res_get__nothrow: true,
   clock_res_get__sig: 'iii',
-  clock_res_get__deps: ['emscripten_get_now', 'emscripten_get_now_res', 'emscripten_get_now_is_monotonic', '$checkWasiClock'],
+  clock_res_get__deps: ['emscripten_get_now', 'emscripten_get_now_res', '$nowIsMonotonic', '$checkWasiClock'],
   clock_res_get: function(clk_id, pres) {
     if (!checkWasiClock(clk_id)) {
       return {{{ cDefine('EINVAL') }}};
@@ -158,7 +170,7 @@ var WasiLibrary = {
     // all wasi clocks but realtime are monotonic
     if (clk_id === {{{ cDefine('CLOCK_REALTIME') }}}) {
       nsec = 1000 * 1000; // educated guess that it's milliseconds
-    } else if (_emscripten_get_now_is_monotonic) {
+    } else if (nowIsMonotonic) {
       nsec = _emscripten_get_now_res();
     } else {
       return {{{ cDefine('ENOSYS') }}};
@@ -171,7 +183,9 @@ var WasiLibrary = {
 #if SYSCALLS_REQUIRE_FILESYSTEM == 0 && (!MINIMAL_RUNTIME || EXIT_RUNTIME)
   $flush_NO_FILESYSTEM: function() {
     // flush anything remaining in the buffers during shutdown
-    if (typeof _fflush !== 'undefined') _fflush(0);
+#if hasExportedFunction('___stdio_exit')
+    ___stdio_exit();
+#endif
     var buffers = SYSCALLS.buffers;
     if (buffers[1].length) SYSCALLS.printChar(1, {{{ charCode("\n") }}});
     if (buffers[2].length) SYSCALLS.printChar(2, {{{ charCode("\n") }}});
@@ -183,6 +197,7 @@ var WasiLibrary = {
 #endif
   fd_write__sig: 'iiiii',
   fd_write: function(fd, iov, iovcnt, pnum) {
+    {{{ from64(['iov', 'iovcnt', 'pnum']) }}};
 #if SYSCALLS_REQUIRE_FILESYSTEM
     var stream = SYSCALLS.getStreamFromFD(fd);
     var num = SYSCALLS.doWritev(stream, iov, iovcnt);
@@ -190,15 +205,16 @@ var WasiLibrary = {
     // hack to support printf in SYSCALLS_REQUIRE_FILESYSTEM=0
     var num = 0;
     for (var i = 0; i < iovcnt; i++) {
-      var ptr = {{{ makeGetValue('iov', 'i*8', 'i32') }}};
-      var len = {{{ makeGetValue('iov', 'i*8 + 4', 'i32') }}};
+      var ptr = {{{ makeGetValue('iov', C_STRUCTS.iovec.iov_base, '*') }}};
+      var len = {{{ makeGetValue('iov', C_STRUCTS.iovec.iov_len, '*') }}};
+      iov += {{{ C_STRUCTS.iovec.__size__ }}};
       for (var j = 0; j < len; j++) {
         SYSCALLS.printChar(fd, HEAPU8[ptr+j]);
       }
       num += len;
     }
 #endif // SYSCALLS_REQUIRE_FILESYSTEM
-    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}}
+    {{{ makeSetValue('pnum', 0, 'num', SIZE_TYPE) }}};
     return 0;
   },
 
@@ -209,7 +225,7 @@ var WasiLibrary = {
     assert(!offset_high, 'offsets over 2^32 not yet supported');
 #endif
     var num = SYSCALLS.doWritev(stream, iov, iovcnt, offset_low);
-    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}}
+    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}};
     return 0;
   },
 
@@ -218,19 +234,15 @@ var WasiLibrary = {
 #if SYSCALLS_REQUIRE_FILESYSTEM
     var stream = SYSCALLS.getStreamFromFD(fd);
     FS.close(stream);
-#else
-#if PROXY_POSIX_SOCKETS
+#elif PROXY_POSIX_SOCKETS
     // close() is a tricky function because it can be used to close both regular file descriptors
     // and POSIX network socket handles, hence an implementation would need to track for each
     // file descriptor which kind of item it is. To simplify, when using PROXY_POSIX_SOCKETS
     // option, use shutdown() to close a socket, and this function should behave like a no-op.
     warnOnce('To close sockets with PROXY_POSIX_SOCKETS bridge, prefer to use the function shutdown() that is proxied, instead of close()')
-#else
-#if ASSERTIONS
+#elif ASSERTIONS
     abort('it should not be possible to operate on streams when !SYSCALLS_REQUIRE_FILESYSTEM');
-#endif
-#endif
-#endif
+#endif // SYSCALLS_REQUIRE_FILESYSTEM
     return 0;
   },
 
@@ -238,7 +250,7 @@ var WasiLibrary = {
   fd_read: function(fd, iov, iovcnt, pnum) {
     var stream = SYSCALLS.getStreamFromFD(fd);
     var num = SYSCALLS.doReadv(stream, iov, iovcnt);
-    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}}
+    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}};
     return 0;
   },
 
@@ -249,7 +261,7 @@ var WasiLibrary = {
 #endif
     var stream = SYSCALLS.getStreamFromFD(fd)
     var num = SYSCALLS.doReadv(stream, iov, iovcnt, offset_low);
-    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}}
+    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}};
     return 0;
   },
 
