@@ -8,6 +8,7 @@
 // specific to Emscripten (and hence not in utility.js).
 
 const FOUR_GB = 4 * 1024 * 1024 * 1024;
+const FLOAT_TYPES = new Set(['float', 'double']);
 
 let currentlyParsedFilename = '';
 
@@ -117,6 +118,14 @@ function preprocess(text, filenameHint) {
             } else if (first === '#endif') {
               assert(showStack.length > 0);
               showStack.pop();
+            } else if (first === '#warning') {
+              if (showCurrentLine()) {
+                printErr(`${filenameHint}:${i + 1}: #warning ${trimmed.substring(trimmed.indexOf(' ')).trim()}`);
+              }
+            } else if (first === '#error') {
+              if (showCurrentLine()) {
+                error(`${filenameHint}:${i + 1}: #error ${trimmed.substring(trimmed.indexOf(' ')).trim()}`);
+              }
             } else {
               throw new Error(`Unknown preprocessor directive on line ${i}: ``${line}```);
             }
@@ -152,27 +161,6 @@ no matching #endif found (${showStack.length$}' unmatched preprocessing directiv
   }
 }
 
-function removePointing(type, num) {
-  if (num === 0) return type;
-  assert(type.substr(type.length - (num ? num : 1)).replace(/\*/g, '') === '');
-  // , 'Error in removePointing with ' + [type, num, type.substr(type.length-(num ? num : 1))]);
-  return type.substr(0, type.length - (num ? num : 1));
-}
-
-function pointingLevels(type) {
-  if (!type) return 0;
-  let ret = 0;
-  const len1 = type.length - 1;
-  while (type[len1 - ret] && type[len1 - ret] === '*') {
-    ret++;
-  }
-  return ret;
-}
-
-function removeAllPointing(type) {
-  return removePointing(type, pointingLevels(type));
-}
-
 // Returns true if ident is a niceIdent (see toNiceIdent). Also allow () and spaces.
 function isNiceIdent(ident, loose) {
   return /^\(?[$_]+[\w$_\d ]*\)?$/.test(ident);
@@ -185,48 +173,18 @@ function needsQuoting(ident) {
   return true;
 }
 
-function isStructPointerType(type) {
-  // This test is necessary for clang - in llvm-gcc, we
-  // could check for %struct. The downside is that %1 can
-  // be either a variable or a structure, and we guess it is
-  // a struct, which can lead to |call i32 %5()| having
-  // |%5()| as a function call (like |i32 (i8*)| etc.). So
-  // we must check later on, in call(), where we have more
-  // context, to differentiate such cases.
-  // A similar thing happens in isStructType()
-  return !Compiletime.isNumberType(type) && type[0] == '%';
-}
+const POINTER_SIZE = MEMORY64 ? 8 : 4;
+const POINTER_BITS = POINTER_SIZE * 8;
+const POINTER_TYPE = 'i' + POINTER_BITS;
+
+const SIZE_TYPE = POINTER_TYPE;
 
 function isPointerType(type) {
   return type[type.length - 1] == '*';
 }
 
-function isArrayType(type) {
-  return /^\[\d+\ x\ (.*)\]/.test(type);
-}
-
-function isStructType(type) {
-  if (isPointerType(type)) return false;
-  if (isArrayType(type)) return true;
-  if (/<?\{ ?[^}]* ?\}>?/.test(type)) return true; // { i32, i8 } etc. - anonymous struct types
-  // See comment in isStructPointerType()
-  return type[0] == '%';
-}
-
-function isVectorType(type) {
-  return type[type.length - 1] === '>';
-}
-
-function isStructuralType(type) {
-  return /^\{ ?[^}]* ?\}$/.test(type); // { i32, i8 } etc. - anonymous struct types
-}
-
-function getStructuralTypeParts(type) { // split { i32, i8 } etc. into parts
-  return type.replace(/[ {}]/g, '').split(',');
-}
-
-function getStructuralTypePartBits(part) {
-  return Math.ceil((getBits(part) || 32) / 32) * 32; // simple 32-bit alignment. || 32 is for pointers
+function pointerT(x) {
+  return MEMORY64 ? `BigInt(${x})` : x;
 }
 
 function isIntImplemented(type) {
@@ -235,170 +193,16 @@ function isIntImplemented(type) {
 
 // Note: works for iX types and structure types, not pointers (even though they are implemented as ints)
 function getBits(type, allowPointers) {
-  if (allowPointers && isPointerType(type)) return 32;
+  if (allowPointers && isPointerType(type)) return POINTER_SIZE;
   if (!type) return 0;
   if (type[0] == 'i') {
     const left = type.substr(1);
     if (!isNumber(left)) return 0;
     return parseInt(left);
   }
-  if (isStructuralType(type)) {
-    return sum(getStructuralTypeParts(type).map(getStructuralTypePartBits));
-  }
-  if (isStructType(type)) {
-    const typeData = Types.types[type];
-    if (typeData === undefined) return 0;
-    return typeData.flatSize * 8;
-  }
   return 0;
 }
 
-function isVoidType(type) {
-  return type == 'void';
-}
-
-// Detects a function definition, ([...|type,[type,...]])
-function isFunctionDef(token, out) {
-  const text = token.text;
-  const nonPointing = removeAllPointing(text);
-  if (nonPointing[0] != '(' || nonPointing.substr(-1) != ')') {
-    return false;
-  }
-  if (nonPointing === '()') return true;
-  if (!token.tokens) return false;
-  let fail = false;
-  const segments = splitTokenList(token.tokens);
-  segments.forEach((segment) => {
-    const subtext = segment[0].text;
-    fail = fail || segment.length > 1 || !(isType(subtext) || subtext == '...');
-  });
-  if (out) {
-    out.segments = segments;
-    out.numArgs = segments.length;
-  }
-  return !fail;
-}
-
-function isPossiblyFunctionType(type) {
-  // A quick but unreliable way to see if something is a function type. Yes is just 'maybe', no is definite.
-  const len = type.length;
-  return type[len - 2] == ')' && type[len - 1] == '*';
-}
-
-function isFunctionType(type, out) {
-  if (!isPossiblyFunctionType(type)) return false;
-  type = type.substr(0, type.length - 1); // remove final '*'
-  const firstOpen = type.indexOf('(');
-  if (firstOpen <= 0) return false;
-  type = type.replace(/"[^"]+"/g, '".."');
-  const lastOpen = type.lastIndexOf('(');
-  let returnType;
-  if (firstOpen == lastOpen) {
-    returnType = getReturnType(type);
-    if (!isType(returnType)) return false;
-  } else {
-    returnType = 'i8*'; // some pointer type, no point in analyzing further
-  }
-  if (out) out.returnType = returnType;
-  // find ( that starts the arguments
-  let depth = 0;
-  let i = type.length - 1;
-  let argText = null;
-  while (i >= 0) {
-    const curr = type[i];
-    if (curr == ')') depth++;
-    else if (curr == '(') {
-      depth--;
-      if (depth == 0) {
-        argText = type.substr(i);
-        break;
-      }
-    }
-    i--;
-  }
-  assert(argText);
-  return isFunctionDef({text: argText, tokens: tokenize(argText.substr(1, argText.length - 2))}, out);
-}
-
-function getReturnType(type) {
-  // the type of a call can be either the return value, or the entire function. ** or more means it is a return value
-  if (pointingLevels(type) > 1) return '*';
-  const lastOpen = type.lastIndexOf('(');
-  if (lastOpen > 0) {
-    // handle things like   void (i32)* (i32, void (i32)*)*
-    const closeStar = type.indexOf(')*');
-    if (closeStar > 0 && closeStar < type.length - 2) lastOpen = closeStar + 3;
-    return type.substr(0, lastOpen - 1);
-  }
-  return type;
-}
-
-const isTypeCache = {}; // quite hot, optimize as much as possible
-
-function isType(type) {
-  if (type in isTypeCache) return isTypeCache[type];
-  const ret = isPointerType(type) || isVoidType(type) || Compiletime.isNumberType(type) || isStructType(type) || isFunctionType(type);
-  isTypeCache[type] = ret;
-  return ret;
-}
-
-const SPLIT_TOKEN_LIST_SPLITTERS = set(',', 'to'); // 'to' can separate parameters as well...
-
-// Splits a list of tokens separated by commas. For example, a list of arguments in a function call
-function splitTokenList(tokens) {
-  if (tokens.length == 0) return [];
-  if (!tokens.slice) tokens = tokens.tokens;
-  const ret = [];
-  const seg = [];
-  for (let i = 0; i < tokens.length; i++) {
-    const token = tokens[i];
-    if (token.text in SPLIT_TOKEN_LIST_SPLITTERS) {
-      ret.push(seg);
-      seg = [];
-    } else if (token.text == ';') {
-      ret.push(seg);
-      return ret;
-    } else {
-      seg.push(token);
-    }
-  }
-  if (seg.length) ret.push(seg);
-  return ret;
-}
-
-function _IntToHex(x) {
-  assert(x >= 0 && x <= 15);
-  if (x <= 9) {
-    return String.fromCharCode('0'.charCodeAt(0) + x);
-  }
-  return String.fromCharCode('A'.charCodeAt(0) + x - 10);
-}
-
-function IEEEUnHex(stringy) {
-  stringy = stringy.substr(2); // leading '0x';
-  if (stringy.replace(/0/g, '') === '') return 0;
-  while (stringy.length < 16) stringy = '0' + stringy;
-  assert(stringy.length === 16, 'Can only unhex 16-digit double numbers, nothing platform-specific'); // |long double| might cause this
-  const top = eval('0x' + stringy[0]);
-  const neg = !!(top & 8); // sign
-  if (neg) {
-    stringy = _IntToHex(top & ~8) + stringy.substr(1);
-  }
-  let a = eval('0x' + stringy.substr(0, 8)); // top half
-  const b = eval('0x' + stringy.substr(8)); // bottom half
-  let e = a >> ((52 - 32) & 0x7ff); // exponent
-  a = a & 0xfffff;
-  if (e === 0x7ff) {
-    if (a == 0 && b == 0) {
-      return neg ? '-Infinity' : 'Infinity';
-    }
-    return 'NaN';
-  }
-  e -= 1023; // offset
-  const absolute = ((((a | 0x100000) * 1.0) / Math.pow(2, 52 - 32)) * Math.pow(2, e)) + (((b * 1.0) / Math.pow(2, 52)) * Math.pow(2, e));
-  return (absolute * (neg ? -1 : 1)).toString();
-}
-
 // Given an expression like (VALUE=VALUE*2,VALUE<10?VALUE:t+1) , this will
 // replace VALUE with value. If value is not a simple identifier of a variable,
 // value will be replaced with tempVar.
@@ -463,7 +267,7 @@ function splitI64(value, floatConversion) {
 function indentify(text, indent) {
   // Don't try to indentify huge strings - we may run out of memory
   if (text.length > 1024 * 1024) return text;
-  if (typeof indent === 'number') {
+  if (typeof indent == 'number') {
     const len = indent;
     indent = '';
     for (let i = 0; i < len; i++) {
@@ -480,7 +284,7 @@ function checkSafeHeap() {
 }
 
 function getHeapOffset(offset, type) {
-  if (Runtime.getNativeFieldSize(type) > 4 && type == 'i64') {
+  if (!WASM_BIGINT && Runtime.getNativeFieldSize(type) > 4 && type == 'i64') {
     // we emulate 64-bit integer values as 32 in asmjs-unknown-emscripten, but not double
     type = 'i32';
   }
@@ -511,7 +315,7 @@ function asmEnsureFloat(value, type) {
     value = ensureDot(value);
     return 'Math.fround(' + value + ')';
   }
-  if (type in Compiletime.FLOAT_TYPES) {
+  if (FLOAT_TYPES.has(type)) {
     return ensureDot(value);
   }
   return value;
@@ -520,7 +324,7 @@ function asmEnsureFloat(value, type) {
 function asmCoercion(value, type, signedness) {
   if (type == 'void') {
     return value;
-  } else if (type in Compiletime.FLOAT_TYPES) {
+  } else if (FLOAT_TYPES.has(type)) {
     if (isNumber(value)) {
       return asmEnsureFloat(value, type);
     } else {
@@ -572,14 +376,6 @@ function makeSetTempDouble(i, type, value) {
 // See makeSetValue
 function makeGetValue(ptr, pos, type, noNeedFirst, unsigned, ignore, align, noSafe, forceAsm) {
   assert(!forceAsm, 'forceAsm is no longer supported');
-  if (isStructType(type)) {
-    const typeData = Types.types[type];
-    const ret = [];
-    for (let i = 0; i < typeData.fields.length; i++) {
-      ret.push('f' + i + ': ' + makeGetValue(ptr, pos + typeData.flatIndexes[i], typeData.fields[i], noNeedFirst, unsigned, 0, 0, noSafe));
-    }
-    return '{ ' + ret.join(', ') + ' }';
-  }
 
   if (type == 'double' && (align < 8)) {
     const setdouble1 = makeSetTempDouble(0, 'i32', makeGetValue(ptr, pos, 'i32', noNeedFirst, unsigned, ignore, align, noSafe));
@@ -620,10 +416,19 @@ function makeGetValue(ptr, pos, type, noNeedFirst, unsigned, ignore, align, noSa
   const offset = calcFastOffset(ptr, pos, noNeedFirst);
   if (SAFE_HEAP && !noSafe) {
     if (!ignore) {
-      return asmCoercion('SAFE_HEAP_LOAD' + ((type in Compiletime.FLOAT_TYPES) ? '_D' : '') + '(' + asmCoercion(offset, 'i32') + ', ' + Runtime.getNativeTypeSize(type) + ', ' + (!!unsigned + 0) + ')', type, unsigned ? 'u' : undefined);
+      return asmCoercion('SAFE_HEAP_LOAD' + (FLOAT_TYPES.has(type) ? '_D' : '') + '(' + asmCoercion(offset, 'i32') + ', ' + Runtime.getNativeTypeSize(type) + ', ' + (!!unsigned + 0) + ')', type, unsigned ? 'u' : undefined);
     }
   }
-  return getHeapForType(type, unsigned) + '[' + getHeapOffset(offset, type) + ']';
+
+  const slab = getHeapForType(type, unsigned);
+  let ret = slab + '[' + getHeapOffset(offset, type) + ']';
+  if (slab.substr(slab.length - 2) == '64') {
+    ret = `Number(${ret})`;
+  }
+  if (forceAsm) {
+    ret = asmCoercion(ret, type);
+  }
+  return ret;
 }
 
 /**
@@ -644,28 +449,14 @@ function makeGetValue(ptr, pos, type, noNeedFirst, unsigned, ignore, align, noSa
  * @param {bool} forcedAlign: legacy, ignored.
  * @return {TODO}
  */
-function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep, forcedAlign) {
+function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe, sep = ';', forcedAlign) {
   assert(!forcedAlign, 'forcedAlign is no longer supported');
-  sep = sep || ';';
-  if (isStructType(type)) {
-    const typeData = Types.types[type];
-    const ret = [];
-    // We can receive either an object - an object literal that was in the .ll - or a string,
-    // which is the ident of an aggregate struct
-    if (typeof value === 'string') {
-      value = range(typeData.fields.length).map((i) => value + '.f' + i);
-    }
-    for (let i = 0; i < typeData.fields.length; i++) {
-      ret.push(makeSetValue(ptr, getFastValue(pos, '+', typeData.flatIndexes[i]), value[i], typeData.fields[i], noNeedFirst, 0, 0, noSafe));
-    }
-    return ret.join('; ');
-  }
 
   if (type == 'double' && (align < 8)) {
     return '(' + makeSetTempDouble(0, 'double', value) + ',' +
             makeSetValue(ptr, pos, makeGetTempDouble(0, 'i32'), 'i32', noNeedFirst, ignore, align, noSafe, ',') + ',' +
             makeSetValue(ptr, getFastValue(pos, '+', Runtime.getNativeTypeSize('i32')), makeGetTempDouble(1, 'i32'), 'i32', noNeedFirst, ignore, align, noSafe, ',') + ')';
-  } else if (type == 'i64') {
+  } else if (!WASM_BIGINT && type == 'i64') {
     return '(tempI64 = [' + splitI64(value) + '],' +
             makeSetValue(ptr, pos, 'tempI64[0]', 'i32', noNeedFirst, ignore, align, noSafe, ',') + ',' +
             makeSetValue(ptr, getFastValue(pos, '+', Runtime.getNativeTypeSize('i32')), 'tempI64[1]', 'i32', noNeedFirst, ignore, align, noSafe, ',') + ')';
@@ -702,21 +493,24 @@ function makeSetValue(ptr, pos, value, type, noNeedFirst, ignore, align, noSafe,
   const offset = calcFastOffset(ptr, pos, noNeedFirst);
   if (SAFE_HEAP && !noSafe) {
     if (!ignore) {
-      return 'SAFE_HEAP_STORE' + ((type in Compiletime.FLOAT_TYPES) ? '_D' : '') + '(' + asmCoercion(offset, 'i32') + ', ' + asmCoercion(value, type) + ', ' + Runtime.getNativeTypeSize(type) + ')';
+      return 'SAFE_HEAP_STORE' + (FLOAT_TYPES.has(type) ? '_D' : '') + '(' + asmCoercion(offset, 'i32') + ', ' + asmCoercion(value, type) + ', ' + Runtime.getNativeTypeSize(type) + ')';
     }
   }
-  return getHeapForType(type) + '[' + getHeapOffset(offset, type) + '] = ' + value;
+
+  const slab = getHeapForType(type);
+  if (slab == 'HEAPU64' || slab == 'HEAP64') {
+    value = `BigInt(${value})`;
+  }
+  return slab + '[' + getHeapOffset(offset, type) + '] = ' + value;
 }
 
 const UNROLL_LOOP_MAX = 8;
 
-function makeCopyValues(dest, src, num, type, modifier, align, sep) {
-  sep = sep || ';';
+function makeCopyValues(dest, src, num, type, modifier, align, sep = ';') {
   function unroll(type, num, jump) {
     jump = jump || 1;
-    return range(num).map((i) => {
-      return makeSetValue(dest, i * jump, makeGetValue(src, i * jump, type), type);
-    }).join(sep);
+    const setValues = range(num).map((i) => makeSetValue(dest, i * jump, makeGetValue(src, i * jump, type), type));
+    return setValues.join(sep);
   }
   // If we don't know how to handle this at compile-time, or handling it is best
   // done in a large amount of code, call memcpy
@@ -769,11 +563,11 @@ function getFastValue(a, op, b, type) {
 
   let aNumber = null;
   let bNumber = null;
-  if (typeof a === 'number') {
+  if (typeof a == 'number') {
     aNumber = a;
     a = a.toString();
   } else if (isNumber(a)) aNumber = parseFloat(a);
-  if (typeof b === 'number') {
+  if (typeof b == 'number') {
     bNumber = b;
     b = b.toString();
   } else if (isNumber(b)) bNumber = parseFloat(b);
@@ -815,7 +609,7 @@ function getFastValue(a, op, b, type) {
   if (op === '*') {
     // We can't eliminate where a or b are 0 as that would break things for creating
     // a negative 0.
-    if ((aNumber === 0 || bNumber === 0) && !(type in Compiletime.FLOAT_TYPES)) {
+    if ((aNumber === 0 || bNumber === 0) && !FLOAT_TYPES.has(type)) {
       return '0';
     } else if (aNumber === 1) {
       return b;
@@ -827,7 +621,7 @@ function getFastValue(a, op, b, type) {
         return `(${a}<<${shifts})`;
       }
     }
-    if (!(type in Compiletime.FLOAT_TYPES)) {
+    if (!FLOAT_TYPES.has(type)) {
       // if guaranteed small enough to not overflow into a double, do a normal multiply
       // default is 32-bit multiply for things like getelementptr indexes
       const bits = getBits(type) || 32;
@@ -841,7 +635,7 @@ function getFastValue(a, op, b, type) {
     }
   } else if (op === '/') {
     // careful on floats, since 0*NaN is not 0
-    if (a === '0' && !(type in Compiletime.FLOAT_TYPES)) {
+    if (a === '0' && !FLOAT_TYPES.has(type)) {
       return '0';
     } else if (b === 1) {
       return a;
@@ -862,13 +656,15 @@ function getFastValue(a, op, b, type) {
 
 function calcFastOffset(ptr, pos, noNeedFirst) {
   assert(!noNeedFirst);
+  if (typeof ptr == 'bigint') ptr = Number(ptr);
+  if (typeof pos == 'bigint') pos = Number(pos);
   return getFastValue(ptr, '+', pos, 'i32');
 }
 
 function getHeapForType(type, unsigned) {
   assert(type);
   if (isPointerType(type)) {
-    type = 'i32'; // Hardcoded 32-bit
+    type = POINTER_TYPE;
   }
   switch (type) {
     case 'i1':
@@ -876,13 +672,15 @@ function getHeapForType(type, unsigned) {
       return unsigned ? 'HEAPU8' : 'HEAP8';
     case 'i16':
       return unsigned ? 'HEAPU16' : 'HEAP16';
-    case '<4 x i32>':
-    case 'i32':
     case 'i64':
+      if (WASM_BIGINT) {
+        return unsigned ? 'HEAPU64' : 'HEAP64';
+      }
+      // fall through
+    case 'i32':
       return unsigned ? 'HEAPU32' : 'HEAP32';
     case 'double':
       return 'HEAPF64';
-    case '<4 x float>':
     case 'float':
       return 'HEAPF32';
   }
@@ -899,7 +697,7 @@ function makeStructuralReturn(values) {
 
 function makeThrow(what) {
   if (ASSERTIONS && DISABLE_EXCEPTION_CATCHING) {
-    what += ' + " - Exception catching is disabled, this exception cannot be caught. Compile with -s NO_DISABLE_EXCEPTION_CATCHING or -s EXCEPTION_CATCHING_ALLOWED=[..] to catch."';
+    what += ' + " - Exception catching is disabled, this exception cannot be caught. Compile with -sNO_DISABLE_EXCEPTION_CATCHING or -sEXCEPTION_CATCHING_ALLOWED=[..] to catch."';
     if (MAIN_MODULE) {
       what += ' + " (note: in dynamic linking, if a side module wants exceptions, the main module must be built with that support)"';
     }
@@ -908,10 +706,7 @@ function makeThrow(what) {
 }
 
 function makeSignOp(value, type, op, force, ignore) {
-  if (type == 'i64') {
-    return value; // these are always assumed to be two 32-bit unsigneds.
-  }
-  if (isPointerType(type)) type = 'i32'; // Pointers are treated as 32-bit ints
+  if (isPointerType(type)) type = POINTER_TYPE;
   if (!value) return value;
   let bits;
   let full;
@@ -1035,13 +830,28 @@ New syntax is {{{ makeDynCall("${sig}", "funcPtr") }}}(arg1, arg2, ...). \
 Please update to new syntax.`);
 
     if (DYNCALLS) {
+      if (!hasExportedFunction(`dynCall_${sig}`)) {
+        if (ASSERTIONS) {
+          return `(function(${args}) { throw 'Internal Error! Attempted to invoke wasm function pointer with signature "${sig}", but no such functions have gotten exported!'; })`;
+        } else {
+          return `(function(${args}) { /* a dynamic function call to signature ${sig}, but there are no exported function pointers with that signature, so this path should never be taken. Build with ASSERTIONS enabled to validate. */ })`;
+        }
+      }
       return `(function(cb, ${args}) { ${returnExpr} getDynCaller("${sig}", cb)(${args}) })`;
     } else {
-      return `(function(cb, ${args}) { ${returnExpr} wasmTable.get(cb)(${args}) })`;
+      return `(function(cb, ${args}) { ${returnExpr} getWasmTableEntry(cb)(${args}) })`;
     }
   }
 
   if (DYNCALLS) {
+    if (!hasExportedFunction(`dynCall_${sig}`)) {
+      if (ASSERTIONS) {
+        return `(function(${args}) { throw 'Internal Error! Attempted to invoke wasm function pointer with signature "${sig}", but no such functions have gotten exported!'; })`;
+      } else {
+        return `(function(${args}) { /* a dynamic function call to signature ${sig}, but there are no exported function pointers with that signature, so this path should never be taken. Build with ASSERTIONS enabled to validate. */ })`;
+      }
+    }
+
     const dyncall = exportedAsmFunc(`dynCall_${sig}`);
     if (sig.length > 1) {
       return `(function(${args}) { ${returnExpr} ${dyncall}.apply(null, [${funcPtr}, ${args}]); })`;
@@ -1049,7 +859,7 @@ Please update to new syntax.`);
       return `(function() { ${returnExpr} ${dyncall}.call(null, ${funcPtr}); })`;
     }
   } else {
-    return `wasmTable.get(${funcPtr})`;
+    return `getWasmTableEntry(${funcPtr})`;
   }
 }
 
@@ -1096,22 +906,20 @@ function addAtExit(code) {
 }
 
 function makeRetainedCompilerSettings() {
-  const ignore = set('STRUCT_INFO');
+  const ignore = new Set(['STRUCT_INFO']);
   if (STRICT) {
     for (const setting of LEGACY_SETTINGS) {
-      const name = setting[0];
-      ignore[name] = 0;
+      ignore.add(setting);
     }
   }
 
   const ret = {};
   for (const x in global) {
-    if (!(x in ignore) && x[0] !== '_' && x == x.toUpperCase()) {
-      try {
-        if (typeof global[x] === 'number' || typeof global[x] === 'string' || this.isArray()) {
-          ret[x] = global[x];
-        }
-      } catch (e) {}
+    if (!ignore.has(x) && x[0] !== '_' && x == x.toUpperCase()) {
+      const value = global[x];
+      if (typeof value == 'number' || typeof value == 'boolean' || typeof value == 'string' || Array.isArray(x)) {
+        ret[x] = value;
+      }
     }
   }
   return ret;
@@ -1121,36 +929,43 @@ function makeRetainedCompilerSettings() {
 const WASM_PAGE_SIZE = 65536;
 
 // Receives a function as text, and a function that constructs a modified
-// function, to which we pass the parsed-out name, arguments, and body of the
-// function. Returns the output of that function.
+// function, to which we pass the parsed-out name, arguments, body, and possible
+// "async" prefix of the input function. Returns the output of that function.
 function modifyFunction(text, func) {
   // Match a function with a name.
-  let match = text.match(/^\s*function\s+([^(]*)?\s*\(([^)]*)\)/);
+  let match = text.match(/^\s*(async\s+)?function\s+([^(]*)?\s*\(([^)]*)\)/);
+  let async_;
   let names;
   let args;
   let rest;
   if (match) {
-    name = match[1];
-    args = match[2];
+    async_ = match[1] || '';
+    name = match[2];
+    args = match[3];
     rest = text.substr(match[0].length);
   } else {
     // Match a function without a name (we could probably use a single regex
     // for both, but it would be more complex).
-    match = text.match(/^\s*function\(([^)]*)\)/);
+    match = text.match(/^\s*(async\s+)?function\(([^)]*)\)/);
     assert(match, 'could not match function ' + text + '.');
     name = '';
-    args = match[1];
+    async_ = match[1] || '';
+    args = match[2];
     rest = text.substr(match[0].length);
   }
   const bodyStart = rest.indexOf('{');
   assert(bodyStart >= 0);
   const bodyEnd = rest.lastIndexOf('}');
   assert(bodyEnd > 0);
-  return func(name, args, rest.substring(bodyStart + 1, bodyEnd));
+  return func(name, args, rest.substring(bodyStart + 1, bodyEnd), async_);
 }
 
 function runOnMainThread(text) {
-  if (USE_PTHREADS) {
+  if (WASM_WORKERS && USE_PTHREADS) {
+    return 'if (!ENVIRONMENT_IS_WASM_WORKER && !ENVIRONMENT_IS_PTHREAD) { ' + text + ' }';
+  } else if (WASM_WORKERS) {
+    return 'if (!ENVIRONMENT_IS_WASM_WORKER) { ' + text + ' }';
+  } else if (USE_PTHREADS) {
     return 'if (!ENVIRONMENT_IS_PTHREAD) { ' + text + ' }';
   } else {
     return text;
@@ -1158,28 +973,25 @@ function runOnMainThread(text) {
 }
 
 function expectToReceiveOnModule(name) {
-  return name in INCOMING_MODULE_JS_API;
+  return INCOMING_MODULE_JS_API.has(name);
 }
 
 function makeRemovedModuleAPIAssert(moduleName, localName) {
   if (!ASSERTIONS) return '';
   if (!localName) localName = moduleName;
-  return `
-if (!Object.getOwnPropertyDescriptor(Module, '${moduleName}')) {
-  Object.defineProperty(Module, '${moduleName}', {
-    configurable: true,
-    get: function() {
-      abort('Module.${moduleName} has been replaced with plain ${localName}\
- (the initial value can be provided on Module,\
- but after startup the value is only looked for on a local variable of that name)')
-    }
-  });
-}`;
+  return `legacyModuleProp('${moduleName}', '${localName}');`;
+}
+
+function checkReceiving(name) {
+  // ALL_INCOMING_MODULE_JS_API contains all valid incoming module API symbols
+  // so calling makeModuleReceive* with a symbol not in this list is an error
+  assert(ALL_INCOMING_MODULE_JS_API.includes(name));
 }
 
 // Make code to receive a value on the incoming Module object.
 function makeModuleReceive(localName, moduleName) {
   if (!moduleName) moduleName = localName;
+  checkReceiving(moduleName);
   let ret = '';
   if (expectToReceiveOnModule(moduleName)) {
     // Usually the local we use is the same as the Module property name,
@@ -1190,8 +1002,18 @@ function makeModuleReceive(localName, moduleName) {
   return ret;
 }
 
+function makeModuleReceiveExpr(name, defaultValue) {
+  checkReceiving(name);
+  if (expectToReceiveOnModule(name)) {
+    return `Module['${name}'] || ${defaultValue}`;
+  } else {
+    return `${defaultValue}`;
+  }
+}
+
 function makeModuleReceiveWithVar(localName, moduleName, defaultValue, noAssert) {
   if (!moduleName) moduleName = localName;
+  checkReceiving(moduleName);
   let ret = 'var ' + localName;
   if (!expectToReceiveOnModule(moduleName)) {
     if (defaultValue) {
@@ -1255,22 +1077,25 @@ function _asmjsDemangle(symbol) {
   if (symbol in WASM_SYSTEM_EXPORTS) {
     return symbol;
   }
+  if (symbol.startsWith('dynCall_')) {
+    return symbol;
+  }
   // Strip leading "_"
   assert(symbol.startsWith('_'));
   return symbol.substr(1);
 }
 
 function hasExportedFunction(func) {
-  return Object.keys(WASM_EXPORTS).includes(_asmjsDemangle(func));
+  return WASM_EXPORTS.has(_asmjsDemangle(func));
 }
 
 // JS API I64 param handling: if we have BigInt support, the ABI is simple,
 // it is a BigInt. Otherwise, we legalize into pairs of i32s.
 function defineI64Param(name) {
   if (WASM_BIGINT) {
-    return name + '_bigint';
+    return `/** @type {!BigInt} */ ${name}_bigint`;
   }
-  return name + '_low, ' + name + '_high';
+  return `${name}_low, ${name}_high`;
 }
 
 function receiveI64ParamAsI32s(name) {
@@ -1285,6 +1110,19 @@ function receiveI64ParamAsI32s(name) {
   return '';
 }
 
+// TODO: use this in library_wasi.js and other places. but we need to add an
+//       error-handling hook here.
+function receiveI64ParamAsDouble(name) {
+  if (WASM_BIGINT) {
+    // Just convert the bigint into a double.
+    return `var ${name} = Number(${name}_bigint);`;
+  }
+
+  // Combine the i32 params. Use an unsigned operator on low and shift high by
+  // 32 bits.
+  return `var ${name} = ${name}_high * 0x100000000 + (${name}_low >>> 0);`;
+}
+
 function sendI64Argument(low, high) {
   if (WASM_BIGINT) {
     return 'BigInt(low) | (BigInt(high) << BigInt(32))';
@@ -1292,6 +1130,26 @@ function sendI64Argument(low, high) {
   return low + ', ' + high;
 }
 
+// Any function called from wasm64 may have bigint args, this function takes
+// a list of variable names to convert to number.
+function from64(x) {
+  if (!MEMORY64) {
+    return '';
+  }
+  if (Array.isArray(x)) {
+    let ret = '';
+    for (e of x) ret += from64(e);
+    return ret;
+  } else {
+    return `${x} = Number(${x});`;
+  }
+}
+
+function to64(x) {
+  if (!MEMORY64) return x;
+  return `BigInt(${x})`;
+}
+
 // Add assertions to catch common errors when using the Promise object we
 // create on Module.ready() and return from MODULARIZE Module() invocations.
 function addReadyPromiseAssertions(promise) {
@@ -1300,7 +1158,7 @@ function addReadyPromiseAssertions(promise) {
   //  var instance = Module();
   //  ...
   //  instance._main();
-  const properties = keys(EXPORTED_FUNCTIONS);
+  const properties = Array.from(EXPORTED_FUNCTIONS.values());
   // Also warn on onRuntimeInitialized which might be a common pattern with
   // older MODULARIZE-using codebases.
   properties.push('onRuntimeInitialized');
@@ -1347,3 +1205,23 @@ function runtimeKeepalivePop() {
   if (MINIMAL_RUNTIME || (EXIT_RUNTIME == 0 && USE_PTHREADS == 0)) return '';
   return 'runtimeKeepalivePop();';
 }
+
+// Some web functions like TextDecoder.decode() may not work with a view of a
+// SharedArrayBuffer, see https://github.com/whatwg/encoding/issues/172
+// To avoid that, this function allows obtaining an unshared copy of an
+// ArrayBuffer.
+function getUnsharedTextDecoderView(heap, start, end) {
+  const shared = `${heap}.slice(${start}, ${end})`;
+  const unshared = `${heap}.subarray(${start}, ${end})`;
+
+  // No need to worry about this in non-shared memory builds
+  if (!SHARED_MEMORY) return unshared;
+
+  // If asked to get an unshared view to what we know will be a shared view, or if in -Oz,
+  // then unconditionally do a .slice() for smallest code size.
+  if (SHRINK_LEVEL == 2 || heap == 'HEAPU8') return shared;
+
+  // Otherwise, generate a runtime type check: must do a .slice() if looking at a SAB,
+  // or can use .subarray() otherwise.
+  return `${heap}.buffer instanceof SharedArrayBuffer ? ${shared} : ${unshared}`;
+}
