@@ -11,13 +11,14 @@
 #error Including <emscripten/bind.h> requires building with -std=c++11 or newer!
 #endif
 
-#include <stddef.h>
-#include <assert.h>
-#include <string>
+#include <cassert>
+#include <cstddef>
 #include <functional>
-#include <vector>
 #include <map>
+#include <string>
 #include <type_traits>
+#include <vector>
+
 #include <emscripten/val.h>
 #include <emscripten/wire.h>
 
@@ -1053,7 +1054,7 @@ struct smart_ptr_trait<std::shared_ptr<PointeeType>> {
         return sharing_policy::BY_EMVAL;
     }
 
-    static std::shared_ptr<PointeeType>* share(PointeeType* p, internal::EM_VAL v) {
+    static std::shared_ptr<PointeeType>* share(PointeeType* p, EM_VAL v) {
         return new std::shared_ptr<PointeeType>(
             p,
             val_deleter(val::take_ownership(v)));
@@ -1205,7 +1206,7 @@ val wrapped_extend(const std::string& name, const val& properties) {
     return val::take_ownership(_embind_create_inheriting_constructor(
         name.c_str(),
         TypeID<WrapperType>::get(),
-        properties.__get_handle()));
+        properties.as_handle()));
 }
 
 } // end namespace internal
@@ -1884,10 +1885,9 @@ void constant(const char* name, const ConstantType& v) {
         static_cast<double>(asGenericValue(BT::toWireType(v))));
 }
 
-} // end namespace emscripten
+// EMSCRIPTEN_BINDINGS simple creates a static constructor function which
+// will get included in the program if the translation unit in which it is
+// define gets linked into the program.
+#define EMSCRIPTEN_BINDINGS(name) __attribute__((constructor)) static void __embind_init_##name(void)
 
-#define EMSCRIPTEN_BINDINGS(name)                                       \
-    static struct EmscriptenBindingInitializer_##name {                 \
-        EmscriptenBindingInitializer_##name();                          \
-    } EmscriptenBindingInitializer_##name##_instance;                   \
-    EmscriptenBindingInitializer_##name::EmscriptenBindingInitializer_##name()
+} // end namespace emscripten
