@@ -15,6 +15,7 @@ import hashlib
 import logging
 import multiprocessing
 import os
+import re
 import shlex
 import shutil
 import stat
@@ -32,15 +33,21 @@ from tools.shared import TEMP_DIR, EMCC, EMXX, DEBUG, EMCONFIGURE, EMCMAKE
 from tools.shared import EMSCRIPTEN_TEMP_DIR
 from tools.shared import EM_BUILD_VERBOSE
 from tools.shared import get_canonical_temp_dir, try_delete, path_from_root
-from tools.utils import MACOS, WINDOWS
+from tools.utils import MACOS, WINDOWS, read_file, read_binary, write_file, write_binary
 from tools import shared, line_endings, building, config
 
 logger = logging.getLogger('common')
 
 # User can specify an environment variable EMTEST_BROWSER to force the browser
 # test suite to run using another browser command line than the default system
-# browser.  Setting '0' as the browser disables running a browser (but we still
-# see tests compile)
+# browser.
+# There are two special value that can be used here if running in an actual
+# browser is not desired:
+#  EMTEST_BROWSER=0 : This will disable the actual running of the test and simply
+#                     verify that it compiles and links.
+#  EMTEST_BROWSER=node : This will attempt to run the browser test under node.
+#                        For most browser tests this does not work, but it can
+#                        be useful for running pthread tests under node.
 EMTEST_BROWSER = None
 EMTEST_DETECT_TEMPFILE_LEAKS = None
 EMTEST_SAVE_DIR = None
@@ -51,6 +58,7 @@ EMTEST_SKIP_SLOW = None
 EMTEST_LACKS_NATIVE_CLANG = None
 EMTEST_VERBOSE = None
 EMTEST_REBASELINE = None
+EMTEST_FORCE64 = None
 
 # Special value for passing to assert_returncode which means we expect that program
 # to fail with non-zero return code, but we don't care about specifically which one.
@@ -62,11 +70,14 @@ WEBIDL_BINDER = shared.bat_suffix(path_from_root('tools/webidl_binder'))
 
 EMBUILDER = shared.bat_suffix(path_from_root('embuilder'))
 EMMAKE = shared.bat_suffix(path_from_root('emmake'))
+WASM_DIS = Path(building.get_binaryen_bin(), 'wasm-dis')
 
 
 def delete_contents(pathname):
   for entry in os.listdir(pathname):
     try_delete(os.path.join(pathname, entry))
+    # TODO(sbc): Should we make try_delete have a stronger guarantee?
+    assert not os.path.exists(os.path.join(pathname, entry))
 
 
 def test_file(*path_components):
@@ -74,14 +85,6 @@ def test_file(*path_components):
   return str(Path(TEST_ROOT, *path_components))
 
 
-def read_file(*path_components):
-  return Path(*path_components).read_text()
-
-
-def read_binary(*path_components):
-  return Path(*path_components).read_bytes()
-
-
 # checks if browser testing is enabled
 def has_browser():
   return EMTEST_BROWSER != '0'
@@ -157,6 +160,7 @@ def no_windows(note=''):
 def requires_native_clang(func):
   assert callable(func)
 
+  @wraps(func)
   def decorated(self, *args, **kwargs):
     if EMTEST_LACKS_NATIVE_CLANG:
       return self.skipTest('native clang tests are disabled')
@@ -187,12 +191,7 @@ def require_v8(func):
 
 def node_pthreads(f):
   def decorated(self, *args, **kwargs):
-    self.set_setting('USE_PTHREADS')
-    self.emcc_args += ['-Wno-pthreads-mem-growth']
-    if self.get_setting('MINIMAL_RUNTIME'):
-      self.skipTest('node pthreads not yet supported with MINIMAL_RUNTIME')
-    self.js_engines = [config.NODE_JS]
-    self.node_args += ['--experimental-wasm-threads', '--experimental-wasm-bulk-memory']
+    self.setup_node_pthreads()
     f(self, *args, **kwargs)
   return decorated
 
@@ -220,14 +219,31 @@ def env_modify(updates):
 
 # Decorator version of env_modify
 def with_env_modify(updates):
+  assert not callable(updates)
+
   def decorated(f):
-    def modified(self):
+    def modified(self, *args, **kwargs):
       with env_modify(updates):
-        return f(self)
+        return f(self, *args, **kwargs)
     return modified
+
   return decorated
 
 
+def also_with_minimal_runtime(f):
+  assert callable(f)
+
+  def metafunc(self, with_minimal_runtime):
+    assert self.get_setting('MINIMAL_RUNTIME') is None
+    if with_minimal_runtime:
+      self.set_setting('MINIMAL_RUNTIME', 1)
+    f(self)
+
+  metafunc._parameterize = {'': (False,),
+                            'minimal_runtime': (True,)}
+  return metafunc
+
+
 def ensure_dir(dirname):
   dirname = Path(dirname)
   dirname.mkdir(parents=True, exist_ok=True)
@@ -375,8 +391,19 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
         self.skipTest('test requires node and EMTEST_SKIP_NODE is set')
       else:
         self.fail('node required to run this test.  Use EMTEST_SKIP_NODE to skip')
+    if self.get_setting('MEMORY64') == 1:
+      self.skipTest("MEMORY64=1 tests don't yet run under node")
     self.js_engines = [config.NODE_JS]
 
+  def setup_node_pthreads(self):
+    self.require_node()
+    self.set_setting('USE_PTHREADS')
+    self.emcc_args += ['-Wno-pthreads-mem-growth']
+    if self.get_setting('MINIMAL_RUNTIME'):
+      self.skipTest('node pthreads not yet supported with MINIMAL_RUNTIME')
+    self.js_engines = [config.NODE_JS]
+    self.node_args += ['--experimental-wasm-threads', '--experimental-wasm-bulk-memory']
+
   def uses_memory_init_file(self):
     if self.get_setting('SIDE_MODULE') or (self.is_wasm() and not self.get_setting('WASM2JS')):
       return False
@@ -402,8 +429,20 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
   def setUp(self):
     super().setUp()
     self.settings_mods = {}
-    self.emcc_args = ['-Werror']
-    self.node_args = ['--stack-trace-limit=50', '--unhandled-rejections=throw']
+    self.emcc_args = ['-Werror', '-Wno-limited-postlink-optimizations']
+    # We want to be strict about closure warnings in our test code.
+    # TODO(sbc): Remove this if we make it the default for `-Werror`:
+    # https://github.com/emscripten-core/emscripten/issues/16205):
+    self.ldflags = ['-sCLOSURE_WARNINGS=error']
+    self.node_args = [
+      # Increate stack trace limit to maximise usefulness of test failure reports
+      '--stack-trace-limit=50',
+      # Opt in to node v15 default behaviour:
+      # https://nodejs.org/api/cli.html#cli_unhandled_rejections_mode
+      '--unhandled-rejections=throw',
+      # Include backtrace for all uncuaght exceptions (not just Error).
+      '--trace-uncaught',
+    ]
     self.v8_args = []
     self.env = {}
     self.temp_files_before_run = []
@@ -427,8 +466,8 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
           print('Not clearing existing test directory')
         else:
           print('Clearing existing test directory')
-          # Even when EMTEST_SAVE_DIR we still try to start with an empty directoy as many tests
-          # expect this.  EMTEST_SAVE_DIR=2 can be used to keep the old contents for the new test
+          # Even when --save-dir is used we still try to start with an empty directory as many tests
+          # expect this.  --no-clean can be used to keep the old contents for the new test
           # run. This can be useful when iterating on a given test with extra files you want to keep
           # around in the output directory.
           delete_contents(self.working_dir)
@@ -483,6 +522,8 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
   def set_setting(self, key, value=1):
     if value is None:
       self.clear_setting(key)
+    if type(value) == bool:
+      value = int(value)
     self.settings_mods[key] = value
 
   def has_changed_setting(self, key):
@@ -524,8 +565,10 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
   # param @main_file whether this is the main file of the test. some arguments
   #                  (like --pre-js) do not need to be passed when building
   #                  libraries, for example
-  def get_emcc_args(self, main_file=False):
+  def get_emcc_args(self, main_file=False, ldflags=True):
     args = self.serialize_settings() + self.emcc_args
+    if ldflags:
+      args += self.ldflags
     if not main_file:
       for i, arg in enumerate(args):
         if arg in ('--pre-js', '--post-js'):
@@ -541,26 +584,35 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     es_check_env = os.environ.copy()
     es_check_env['PATH'] = os.path.dirname(config.NODE_JS[0]) + os.pathsep + es_check_env['PATH']
     try:
-      shared.run_process(es_check + ['es5', os.path.abspath(filename), '--quiet'], stderr=PIPE, env=es_check_env)
+      # es-check prints the details of the errors to stdout, but it also prints
+      # stuff in the case there are no errors:
+      #  ES-Check: there were no ES version matching errors!
+      # pipe stdout and stderr so that we can choose if/when to print this
+      # output and avoid spamming stdout when tests are successful.
+      shared.run_process(es_check + ['es5', os.path.abspath(filename)], stdout=PIPE, stderr=STDOUT, env=es_check_env)
     except subprocess.CalledProcessError as e:
-      print(e.stderr)
+      print(e.stdout)
       self.fail('es-check failed to verify ES5 output compliance')
 
   # Build JavaScript code from source code
-  def build(self, filename, libraries=[], includes=[], force_c=False, js_outfile=True, emcc_args=[]):
+  def build(self, filename, libraries=[], includes=[], force_c=False, js_outfile=True, emcc_args=[], output_basename=None):
     suffix = '.js' if js_outfile else '.wasm'
     compiler = [compiler_for(filename, force_c)]
     if compiler[0] == EMCC:
       # TODO(https://github.com/emscripten-core/emscripten/issues/11121)
-      # We link with C++ stdlibs, even when linking with emcc for historical reasons.  We can remove
-      # this if this issues is fixed.
-      compiler.append('-nostdlib++')
+      # For historical reasons emcc compiles and links as C++ by default.
+      # However we want to run our tests in a more strict manner.  We can
+      # remove this if the issue above is ever fixed.
+      compiler.append('-sNO_DEFAULT_TO_CXX')
 
     if force_c:
       compiler.append('-xc')
 
-    dirname, basename = os.path.split(filename)
-    output = shared.unsuffixed(basename) + suffix
+    if output_basename:
+      output = output_basename + suffix
+    else:
+      basename = os.path.basename(filename)
+      output = shared.unsuffixed(basename) + suffix
     cmd = compiler + [filename, '-o', output] + self.get_emcc_args(main_file=True) + emcc_args + libraries
     if shared.suffix(filename) not in ('.i', '.ii'):
       # Add the location of the test file to include path.
@@ -569,8 +621,6 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
 
     self.run_process(cmd, stderr=self.stderr_redirect if not DEBUG else None)
     self.assertExists(output)
-    if js_outfile and not self.uses_es6:
-      self.verify_es5(output)
 
     if js_outfile and self.uses_memory_init_file():
       src = read_file(output)
@@ -620,12 +670,17 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     self.fail('Failed to find [%s] in wasm-opt output' % what)
 
   def get_wasm_text(self, wasm_binary):
-    return self.run_process([os.path.join(building.get_binaryen_bin(), 'wasm-dis'), wasm_binary], stdout=PIPE).stdout
+    return self.run_process([WASM_DIS, wasm_binary], stdout=PIPE).stdout
 
   def is_exported_in_wasm(self, name, wasm):
     wat = self.get_wasm_text(wasm)
     return ('(export "%s"' % name) in wat
 
+  def measure_wasm_code_lines(self, wasm):
+    wat_lines = self.get_wasm_text(wasm).splitlines()
+    non_data_lines = [line for line in wat_lines if '(data ' not in line]
+    return len(non_data_lines)
+
   def run_js(self, filename, engine=None, args=[],
              output_nicerizer=None,
              assert_returncode=0,
@@ -639,19 +694,20 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       stderr_file = self.in_dir('stderr')
       stderr = open(stderr_file, 'w')
     error = None
+    timeout_error = None
     if not engine:
       engine = self.js_engines[0]
     if engine == config.NODE_JS:
       engine = engine + self.node_args
     if engine == config.V8_ENGINE:
       engine = engine + self.v8_args
-    if EMTEST_VERBOSE:
-      print(f"Running '{filename}' under '{shared.shlex_join(engine)}'")
     try:
       jsrun.run_js(filename, engine, args,
                    stdout=open(stdout_file, 'w'),
                    stderr=stderr,
                    assert_returncode=assert_returncode)
+    except subprocess.TimeoutExpired as e:
+      timeout_error = e
     except subprocess.CalledProcessError as e:
       error = e
 
@@ -664,7 +720,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       ret += read_file(stderr_file)
     if output_nicerizer:
       ret = output_nicerizer(ret)
-    if error or EMTEST_VERBOSE:
+    if error or timeout_error or EMTEST_VERBOSE:
       ret = limit_size(ret)
       print('-- begin program output --')
       print(read_file(stdout_file), end='')
@@ -673,11 +729,13 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
         print('-- begin program stderr --')
         print(read_file(stderr_file), end='')
         print('-- end program stderr --')
-      if error:
-        if assert_returncode == NON_ZERO:
-          self.fail('JS subprocess unexpectedly succeeded (%s):  Output:\n%s' % (error.cmd, ret))
-        else:
-          self.fail('JS subprocess failed (%s): %s.  Output:\n%s' % (error.cmd, error.returncode, ret))
+    if timeout_error:
+      raise timeout_error
+    if error:
+      if assert_returncode == NON_ZERO:
+        self.fail('JS subprocess unexpectedly succeeded (%s):  Output:\n%s' % (error.cmd, ret))
+      else:
+        self.fail('JS subprocess failed (%s): %s.  Output:\n%s' % (error.cmd, error.returncode, ret))
 
     #  We should pass all strict mode checks
     self.assertNotContained('strict warning:', ret)
@@ -721,7 +779,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       print("Expected to have '%s' == '%s'" % (limit_size(values[0]), limit_size(y)))
     fail_message = 'Unexpected difference:\n' + limit_size(diff)
     if not EMTEST_VERBOSE:
-      fail_message += '\nFor full output run with EMTEST_VERBOSE=1.'
+      fail_message += '\nFor full output run with --verbose.'
     if msg:
       fail_message += '\n' + msg
     self.fail(fail_message)
@@ -731,6 +789,25 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     text2 = text2.replace('\r\n', '\n')
     return self.assertContained(text1, text2)
 
+  def assertFileContents(self, filename, contents):
+    if EMTEST_VERBOSE:
+      print(f'Comparing results contents of file: {filename}')
+
+    contents = contents.replace('\r', '')
+
+    if EMTEST_REBASELINE:
+      with open(filename, 'w') as f:
+        f.write(contents)
+      return
+
+    if not os.path.exists(filename):
+      self.fail('Test expectation file not found: ' + filename + '.\n' +
+                'Run with --rebaseline to generate.')
+    expected_content = read_file(filename)
+    message = "Run with --rebaseline to automatically update expectations"
+    self.assertTextDataIdentical(expected_content, contents, message,
+                                 filename, filename + '.new')
+
   def assertContained(self, values, string, additional_info=''):
     if type(values) not in [list, tuple]:
       values = [values]
@@ -751,10 +828,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     if callable(string):
       string = string()
     if value in string:
-      self.fail("Expected to NOT find '%s' in '%s', diff:\n\n%s" % (
-        limit_size(value), limit_size(string),
-        limit_size(''.join([a.rstrip() + '\n' for a in difflib.unified_diff(value.split('\n'), string.split('\n'), fromfile='expected', tofile='actual')]))
-      ))
+      self.fail("Expected to NOT find '%s' in '%s'" % (limit_size(value), limit_size(string)))
 
   def assertContainedIf(self, value, string, condition):
     if condition:
@@ -786,7 +860,10 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     build_dir = self.get_build_dir()
     output_dir = self.get_dir()
 
-    emcc_args = self.get_emcc_args()
+    # get_library() is used to compile libraries, and not link executables,
+    # so we don't want to pass linker flags here (emscripten warns if you
+    # try to pass linker settings when compiling).
+    emcc_args = self.get_emcc_args(ldflags=False)
 
     hash_input = (str(emcc_args) + ' $ ' + str(env_init)).encode('utf-8')
     cache_name = name + ','.join([opt for opt in emcc_args if len(opt) < 7]) + '_' + hashlib.md5(hash_input).hexdigest() + cache_name_extra
@@ -799,8 +876,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       generated_libs = []
       for basename, contents in self.library_cache[cache_name]:
         bc_file = os.path.join(build_dir, cache_name + '_' + basename)
-        with open(bc_file, 'wb') as f:
-          f.write(contents)
+        write_binary(bc_file, contents)
         generated_libs.append(bc_file)
       return generated_libs
 
@@ -809,7 +885,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       # Avoid += so we don't mutate the default arg
       configure = configure + configure_args
 
-    cflags = ' '.join(self.get_emcc_args())
+    cflags = ' '.join(emcc_args)
     env_init.setdefault('CFLAGS', cflags)
     env_init.setdefault('CXXFLAGS', cflags)
     return build_library(name, build_dir, output_dir, generated_libs, configure,
@@ -835,10 +911,10 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
         self.fail(f'subprocess exited with non-zero return code({e.returncode}): `{shared.shlex_join(cmd)}`')
 
   def emcc(self, filename, args=[], output_filename=None, **kwargs):
-    if output_filename is None:
-      output_filename = filename + '.o'
-    try_delete(output_filename)
-    self.run_process([compiler_for(filename), filename] + args + ['-o', output_filename], **kwargs)
+    cmd = [compiler_for(filename), filename] + args
+    if output_filename:
+      cmd += ['-o', output_filename]
+    self.run_process(cmd, **kwargs)
 
   # Shared test code between main suite and others
 
@@ -924,7 +1000,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     so = '.wasm' if self.is_wasm() else '.js'
 
     def ccshared(src, linkto=[]):
-      cmdv = [EMCC, src, '-o', shared.unsuffixed(src) + so, '-s', 'SIDE_MODULE'] + self.get_emcc_args()
+      cmdv = [EMCC, src, '-o', shared.unsuffixed(src) + so, '-sSIDE_MODULE'] + self.get_emcc_args()
       cmdv += linkto
       self.run_process(cmdv)
 
@@ -1000,12 +1076,11 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
         filename = 'src.c'
       else:
         filename = 'src.cpp'
-      with open(filename, 'w') as f:
-        f.write(src)
+      write_file(filename, src)
     self._build_and_run(filename, expected_output, **kwargs)
 
   def do_runf(self, filename, expected_output=None, **kwargs):
-    self._build_and_run(filename, expected_output, **kwargs)
+    return self._build_and_run(filename, expected_output, **kwargs)
 
   ## Just like `do_run` but with filename of expected output
   def do_run_from_file(self, filename, expected_output_filename, **kwargs):
@@ -1025,14 +1100,17 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
                      includes=[],
                      assert_returncode=0, assert_identical=False, assert_all=False,
                      check_for_error=True, force_c=False, emcc_args=[],
-                     interleaved_output=True):
+                     interleaved_output=True,
+                     regex=False,
+                     output_basename=None):
     logger.debug(f'_build_and_run: {filename}')
 
     if no_build:
       js_file = filename
     else:
       js_file = self.build(filename, libraries=libraries, includes=includes,
-                           force_c=force_c, emcc_args=emcc_args)
+                           force_c=force_c, emcc_args=emcc_args,
+                           output_basename=output_basename)
     self.assertExists(js_file)
 
     engines = self.filtered_js_engines(js_engines)
@@ -1047,8 +1125,8 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       engines += self.wasm_engines
       if self.get_setting('WASM2C') and not EMTEST_LACKS_NATIVE_CLANG:
         # compile the c file to a native executable.
-        c = shared.unsuffixed(js_file) + '.wasm.c'
-        executable = shared.unsuffixed(js_file) + '.exe'
+        c = shared.replace_suffix(js_file, '.wasm.c')
+        executable = shared.replace_suffix(js_file, '.exe')
         cmd = [shared.CLANG_CC, c, '-o', executable] + clang_native.get_clang_native_args()
         self.run_process(cmd, env=clang_native.get_clang_native_env())
         # we can now run the executable directly, without an engine, which
@@ -1066,16 +1144,24 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
         try:
           if assert_identical:
             self.assertIdentical(expected_output, js_output)
-          elif assert_all:
+          elif assert_all or len(expected_output) == 1:
             for o in expected_output:
-              self.assertContained(o, js_output)
+              if regex:
+                self.assertTrue(re.search(o, js_output), 'Expected regex "%s" to match on:\n%s' % (regex, js_output))
+              else:
+                self.assertContained(o, js_output)
           else:
-            self.assertContained(expected_output, js_output)
-            if check_for_error:
+            if regex:
+              match_any = any(re.search(o, js_output) for o in expected_output)
+              self.assertTrue(match_any, 'Expected at least one of "%s" to match on:\n%s' % (expected_output, js_output))
+            else:
+              self.assertContained(expected_output, js_output)
+            if assert_returncode == 0 and check_for_error:
               self.assertNotContained('ERROR', js_output)
         except Exception:
           print('(test did not pass in JS engine: %s)' % engine)
           raise
+    return js_output
 
   def get_freetype_library(self):
     if '-Werror' in self.emcc_args:
@@ -1118,6 +1204,8 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     return poppler + freetype
 
   def get_zlib_library(self):
+    # TODO: remove -Wno-unknown-warning-option when clang rev 11da1b53 rolls into emscripten
+    self.emcc_args += ['-Wno-deprecated-non-prototype', '-Wno-unknown-warning-option']
     if WINDOWS:
       return self.get_library(os.path.join('third_party', 'zlib'), os.path.join('libz.a'),
                               configure=['cmake', '.'],
@@ -1298,9 +1386,9 @@ class BrowserCore(RunnerCore):
   @classmethod
   def setUpClass(cls):
     super().setUpClass()
-    cls.also_asmjs = int(os.getenv('EMTEST_BROWSER_ALSO_ASMJS', '0')) == 1
+    cls.also_wasm2js = int(os.getenv('EMTEST_BROWSER_ALSO_WASM2JS', '0')) == 1
     cls.port = int(os.getenv('EMTEST_BROWSER_PORT', '8888'))
-    if not has_browser():
+    if not has_browser() or EMTEST_BROWSER == 'node':
       return
     cls.browser_timeout = 60
     cls.harness_in_queue = multiprocessing.Queue()
@@ -1313,7 +1401,7 @@ class BrowserCore(RunnerCore):
   @classmethod
   def tearDownClass(cls):
     super().tearDownClass()
-    if not has_browser():
+    if not has_browser() or EMTEST_BROWSER == 'node':
       return
     cls.harness_server.terminate()
     print('[Browser harness server terminated]')
@@ -1398,8 +1486,7 @@ class BrowserCore(RunnerCore):
     basename = os.path.basename(expected)
     shutil.copyfile(expected, os.path.join(self.get_dir(), basename))
     reporting = read_file(test_file('browser_reporting.js'))
-    with open('reftest.js', 'w') as out:
-      out.write('''
+    write_file('reftest.js', '''
       function doReftest() {
         if (doReftest.done) return;
         doReftest.done = true;
@@ -1459,42 +1546,48 @@ class BrowserCore(RunnerCore):
         img.src = '%s';
       };
 
-      // Automatically trigger the reftest?
-      if (!%s) {
-        // Yes, automatically
+      /** @suppress {uselessCode} */
+      function setupRefTest() {
+        // Automatically trigger the reftest?
+        if (!%s) {
+          // Yes, automatically
 
-        Module['postRun'] = doReftest;
+          Module['postRun'] = doReftest;
 
-        if (typeof WebGLClient !== 'undefined') {
-          // trigger reftest from RAF as well, needed for workers where there is no pre|postRun on the main thread
-          var realRAF = window.requestAnimationFrame;
-          window.requestAnimationFrame = /** @suppress{checkTypes} */ (function(func) {
-            realRAF(function() {
-              func();
-              realRAF(doReftest);
-            });
-          });
+          if (typeof WebGLClient !== 'undefined') {
+            // trigger reftest from RAF as well, needed for workers where there is no pre|postRun on the main thread
+            var realRAF = window.requestAnimationFrame;
+            /** @suppress{checkTypes} */
+            window.requestAnimationFrame = function(func) {
+              return realRAF(function() {
+                func();
+                realRAF(doReftest);
+              });
+            };
 
-          // trigger reftest from canvas render too, for workers not doing GL
-          var realWOM = worker.onmessage;
-          worker.onmessage = function(event) {
-            realWOM(event);
-            if (event.data.target === 'canvas' && event.data.op === 'render') {
-              realRAF(doReftest);
-            }
+            // trigger reftest from canvas render too, for workers not doing GL
+            var realWOM = worker.onmessage;
+            worker.onmessage = function(event) {
+              realWOM(event);
+              if (event.data.target === 'canvas' && event.data.op === 'render') {
+                realRAF(doReftest);
+              }
+            };
+          }
+
+        } else {
+          // Manually trigger the reftest.
+
+          // The user will call it.
+          // Add an event loop iteration to ensure rendering, so users don't need to bother.
+          var realDoReftest = doReftest;
+          doReftest = function() {
+            setTimeout(realDoReftest, 1);
           };
         }
-
-      } else {
-        // Manually trigger the reftest.
-
-        // The user will call it.
-        // Add an event loop iteration to ensure rendering, so users don't need to bother.
-        var realDoReftest = doReftest;
-        doReftest = function() {
-          setTimeout(realDoReftest, 1);
-        };
       }
+
+      setupRefTest();
 ''' % (reporting, basename, int(manually_trigger)))
 
   def compile_btest(self, args, reporting=Reporting.FULL):
@@ -1502,17 +1595,19 @@ class BrowserCore(RunnerCore):
     # use REPORT_RESULT, and also adds a cpp file to be compiled alongside the testcase, which
     # contains the implementation of REPORT_RESULT (we can't just include that implementation in
     # the header as there may be multiple files being compiled here).
-    args += ['-s', 'IN_TEST_HARNESS']
+    args += ['-sIN_TEST_HARNESS']
     if reporting != Reporting.NONE:
       # For basic reporting we inject JS helper funtions to report result back to server.
       args += ['-DEMTEST_PORT_NUMBER=%d' % self.port,
                '--pre-js', test_file('browser_reporting.js')]
       if reporting == Reporting.FULL:
         # If C reporting (i.e. REPORT_RESULT macro) is required
-        # also compile in report_result.cpp and forice-include report_result.h
+        # also compile in report_result.c and forice-include report_result.h
         args += ['-I' + TEST_ROOT,
                  '-include', test_file('report_result.h'),
-                 test_file('report_result.cpp')]
+                 test_file('report_result.c')]
+    if EMTEST_BROWSER == 'node':
+      args.append('-DEMTEST_NODE')
     self.run_process([EMCC] + self.get_emcc_args() + args)
 
   def btest_exit(self, filename, assert_returncode=0, *args, **kwargs):
@@ -1523,6 +1618,8 @@ class BrowserCore(RunnerCore):
     REPORT_RESULT macro to the C code.
     """
     self.set_setting('EXIT_RUNTIME')
+    assert('reporting' not in kwargs)
+    assert('expected' not in kwargs)
     kwargs['reporting'] = Reporting.JS_ONLY
     kwargs['expected'] = 'exit:%d' % assert_returncode
     return self.btest(filename, *args, **kwargs)
@@ -1530,7 +1627,7 @@ class BrowserCore(RunnerCore):
   def btest(self, filename, expected=None, reference=None,
             reference_slack=0, manual_reference=False, post_build=None,
             args=None, message='.', also_proxied=False,
-            url_suffix='', timeout=None, also_asmjs=False,
+            url_suffix='', timeout=None, also_wasm2js=False,
             manually_trigger_reftest=False, extra_tries=1,
             reporting=Reporting.FULL):
     assert expected or reference, 'a btest must either expect an output, or have a reference image'
@@ -1545,7 +1642,7 @@ class BrowserCore(RunnerCore):
       expected = [str(i) for i in range(0, reference_slack + 1)]
       self.reftest(test_file(reference), manually_trigger=manually_trigger_reftest)
       if not manual_reference:
-        args += ['--pre-js', 'reftest.js', '-s', 'GL_TESTING']
+        args += ['--pre-js', 'reftest.js', '-sGL_TESTING']
     outfile = 'test.html'
     args += [filename, '-o', outfile]
     # print('all args:', args)
@@ -1556,13 +1653,19 @@ class BrowserCore(RunnerCore):
       post_build()
     if not isinstance(expected, list):
       expected = [expected]
-    self.run_browser(outfile + url_suffix, message, ['/report_result?' + e for e in expected], timeout=timeout, extra_tries=extra_tries)
+    if EMTEST_BROWSER == 'node':
+      self.js_engines = [config.NODE_JS]
+      self.node_args += ['--experimental-wasm-threads', '--experimental-wasm-bulk-memory']
+      output = self.run_js('test.js')
+      self.assertContained('RESULT: ' + expected[0], output)
+    else:
+      self.run_browser(outfile + url_suffix, message, ['/report_result?' + e for e in expected], timeout=timeout, extra_tries=extra_tries)
 
     # Tests can opt into being run under asmjs as well
-    if 'WASM=0' not in original_args and (also_asmjs or self.also_asmjs):
+    if 'WASM=0' not in original_args and (also_wasm2js or self.also_wasm2js):
       print('WASM=0')
       self.btest(filename, expected, reference, reference_slack, manual_reference, post_build,
-                 original_args + ['-s', 'WASM=0'], message, also_proxied=False, timeout=timeout)
+                 original_args + ['-sWASM=0'], message, also_proxied=False, timeout=timeout)
 
     if also_proxied:
       print('proxied...')
@@ -1573,7 +1676,7 @@ class BrowserCore(RunnerCore):
         post_build = self.post_manual_reftest
       # run proxied
       self.btest(filename, expected, reference, reference_slack, manual_reference, post_build,
-                 original_args + ['--proxy-to-worker', '-s', 'GL_TESTING'], message, timeout=timeout)
+                 original_args + ['--proxy-to-worker', '-sGL_TESTING'], message, timeout=timeout)
 
 
 ###################################################################################################
