@@ -6,9 +6,11 @@
 import os
 import sys
 import logging
+
+from . import utils
 from .utils import path_from_root, exit_with_error, __rootpath__, which
 
-logger = logging.getLogger('shared')
+logger = logging.getLogger('config')
 
 # The following class can be overridden by the config file and/or
 # environment variables.  Specifically any variable whose name
@@ -36,9 +38,9 @@ COMPILER_WRAPPER = None
 
 
 def listify(x):
-  if type(x) is not list:
-    return [x]
-  return x
+  if x is None or type(x) is list:
+    return x
+  return [x]
 
 
 def fix_js_engine(old, new):
@@ -54,7 +56,7 @@ def root_is_writable():
 
 
 def normalize_config_settings():
-  global CACHE, PORTS, JAVA, LLVM_ADD_VERSION, CLANG_ADD_VERSION
+  global CACHE, PORTS, LLVM_ADD_VERSION, CLANG_ADD_VERSION, CLOSURE_COMPILER
   global NODE_JS, V8_ENGINE, JS_ENGINE, JS_ENGINES, SPIDERMONKEY_ENGINE, WASM_ENGINES
 
   # EM_CONFIG stuff
@@ -74,6 +76,7 @@ def normalize_config_settings():
   JS_ENGINE = fix_js_engine(JS_ENGINE, listify(JS_ENGINE))
   JS_ENGINES = [listify(engine) for engine in JS_ENGINES]
   WASM_ENGINES = [listify(engine) for engine in WASM_ENGINES]
+  CLOSURE_COMPILER = listify(CLOSURE_COMPILER)
   if not CACHE:
     if FROZEN_CACHE or root_is_writable():
       CACHE = path_from_root('cache')
@@ -88,10 +91,6 @@ def normalize_config_settings():
   if not PORTS:
     PORTS = os.path.join(CACHE, 'ports')
 
-  if JAVA is None:
-    logger.debug('JAVA not defined in ' + EM_CONFIG + ', using "java"')
-    JAVA = 'java'
-
   # Tools/paths
   if LLVM_ADD_VERSION is None:
     LLVM_ADD_VERSION = os.getenv('LLVM_ADD_VERSION')
@@ -106,7 +105,7 @@ def parse_config_file():
   Also check EM_<KEY> environment variables to override specific config keys.
   """
   config = {}
-  config_text = open(EM_CONFIG, 'r').read()
+  config_text = utils.read_file(EM_CONFIG)
   try:
     exec(config_text, config)
   except Exception as e:
@@ -138,12 +137,15 @@ def parse_config_file():
     env_var = 'EM_' + key
     env_value = os.environ.get(env_var)
     if env_value is not None:
+      if env_value == '':
+        env_value = None
       globals()[key] = env_value
     elif key in config:
       globals()[key] = config[key]
 
-  # Handle legacy environment variables that were previously honored by the
-  # default config file.
+  # In the past the default-generated .emscripten config file would read certain environment
+  # variables. We used generate a warning here but that could generates false positives
+  # See https://github.com/emscripten-core/emsdk/issues/862
   LEGACY_ENV_VARS = {
     'LLVM': 'EM_LLVM_ROOT',
     'BINARYEN': 'EM_BINARYEN_ROOT',
@@ -152,8 +154,7 @@ def parse_config_file():
   for key, new_key in LEGACY_ENV_VARS.items():
     env_value = os.environ.get(key)
     if env_value and new_key not in os.environ:
-      logger.warning(f'warning: honoring legacy environment variable `{key}`.  Please switch to using `{new_key}` instead`')
-      globals()[new_key] = env_value
+      logger.debug(f'legacy environment variable found: `{key}`.  Please switch to using `{new_key}` instead`')
 
   # Certain keys are mandatory
   for key in ('LLVM_ROOT', 'NODE_JS', 'BINARYEN_ROOT'):
@@ -168,11 +169,15 @@ def parse_config_file():
   normalize_config_settings()
 
 
-def generate_config(path, first_time=False):
+def generate_config(path):
+  if os.path.exists(path):
+    exit_with_error(f'config file already exists: `{path}`')
+
   # Note: repr is used to ensure the paths are escaped correctly on Windows.
   # The full string is replaced so that the template stays valid Python.
-  config_data = open(path_from_root('tools', 'settings_template.py')).read().splitlines()
-  config_data = config_data[3:] # remove the initial comment
+
+  config_data = utils.read_file(path_from_root('tools/config_template.py'))
+  config_data = config_data.splitlines()[3:] # remove the initial comment
   config_data = '\n'.join(config_data)
   # autodetect some default paths
   config_data = config_data.replace('\'{{{ EMSCRIPTEN_ROOT }}}\'', repr(__rootpath__))
@@ -182,19 +187,13 @@ def generate_config(path, first_time=False):
   node = which('node') or which('nodejs') or 'node'
   config_data = config_data.replace('\'{{{ NODE }}}\'', repr(node))
 
-  abspath = os.path.abspath(os.path.expanduser(path))
   # write
-  with open(abspath, 'w') as f:
-    f.write(config_data)
+  utils.write_file(path, config_data)
 
-  if first_time:
-    print('''
-==============================================================================
-Welcome to Emscripten!
+  print('''\
+An Emscripten settings file has been generated at:
 
-This is the first time any of the Emscripten tools has been run.
-
-A settings file has been copied to %s, at absolute path: %s
+  %s
 
 It contains our best guesses for the important paths, which are:
 
@@ -202,11 +201,8 @@ It contains our best guesses for the important paths, which are:
   NODE_JS         = %s
   EMSCRIPTEN_ROOT = %s
 
-Please edit the file if any of those are incorrect.
-
-This command will now exit. When you are done editing those paths, re-run it.
-==============================================================================
-''' % (path, abspath, llvm_root, node, __rootpath__), file=sys.stderr)
+Please edit the file if any of those are incorrect.\
+''' % (path, llvm_root, node, __rootpath__), file=sys.stderr)
 
 
 # Emscripten configuration is done through the --em-config command line option
@@ -254,20 +250,29 @@ elif os.path.exists(emsdk_embedded_config):
 elif os.path.exists(user_home_config):
   EM_CONFIG = user_home_config
 else:
+  # No config file found.  Set EM_CONFIG to a default value
+  # that will get reported in the error below.
   if root_is_writable():
-    generate_config(embedded_config, first_time=True)
+    EM_CONFIG = embedded_config
   else:
-    generate_config(user_home_config, first_time=True)
-  sys.exit(0)
+    EM_CONFIG = user_home_config
 
 # We used to support inline EM_CONFIG.
 if '\n' in EM_CONFIG:
   exit_with_error('Inline EM_CONFIG data no longer supported.  Please use a config file.')
 
 EM_CONFIG = os.path.expanduser(EM_CONFIG)
-logger.debug('emscripten config is located in ' + EM_CONFIG)
+
+# This command line flag needs to work even in the absence of a config file, so we must process it
+# here at script import time (otherwise the error below will trigger).
+if '--generate-config' in sys.argv:
+  generate_config(EM_CONFIG)
+  sys.exit(0)
+
 if not os.path.exists(EM_CONFIG):
-  exit_with_error('emscripten config file not found: ' + EM_CONFIG)
+  exit_with_error(f'config file not found: {EM_CONFIG}.  Please create one by hand or run `emcc --generate-config`')
+
+logger.debug('emscripten config is located in ' + EM_CONFIG)
 
 # Emscripten compiler spawns other processes, which can reimport shared.py, so
 # make sure that those child processes get the same configuration file by
