@@ -73,8 +73,9 @@ _deps_info = {
   'alcGetString': ['malloc'],
   'bind': ['ntohs', 'htons'],
   'connect': ['ntohs', 'htons'],
-  'ctime': ['_get_tzname', '_get_daylight', '_get_timezone', 'malloc'],
-  'ctime_r': ['_get_tzname', '_get_daylight', '_get_timezone', 'malloc'],
+  'ctime': ['malloc'],
+  'gmtime': ['malloc'],
+  'ctime_r': ['malloc'],
   'dladdr': ['malloc'],
   'dlopen': ['__dl_seterr'],
   'dlsym': ['__dl_seterr'],
@@ -104,55 +105,50 @@ _deps_info = {
   'emscripten_pc_get_function': ['malloc', 'free'],
   'emscripten_run_preload_plugins_data': ['malloc'],
   'emscripten_run_script_string': ['emscripten_builtin_malloc', 'emscripten_builtin_free'],
-  'emscripten_set_batterychargingchange_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_batterylevelchange_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_beforeunload_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_blur_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_click_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_dblclick_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_devicemotion_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_deviceorientation_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_focus_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_focusin_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_focusout_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_fullscreenchange_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_gamepadconnected_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_gamepaddisconnected_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_keydown_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_keypress_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_keyup_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_mousedown_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_mouseenter_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_mouseleave_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_mousemove_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_mouseout_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_mouseover_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_mouseup_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_offscreencanvas_size_on_target_thread_js': ['malloc', 'free'],
-  'emscripten_set_orientationchange_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_pointerlockchange_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_pointerlockerror_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_resize_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_scroll_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_touchcancel_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_touchend_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_touchmove_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_touchstart_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_visibilitychange_callback_on_thread': ['malloc', 'free'],
-  'emscripten_set_wheel_callback_on_thread': ['malloc', 'free'],
+  'emscripten_set_batterychargingchange_callback_on_thread': ['malloc'],
+  'emscripten_set_batterylevelchange_callback_on_thread': ['malloc'],
+  'emscripten_set_blur_callback_on_thread': ['malloc'],
+  'emscripten_set_click_callback_on_thread': ['malloc'],
+  'emscripten_set_dblclick_callback_on_thread': ['malloc'],
+  'emscripten_set_devicemotion_callback_on_thread': ['malloc'],
+  'emscripten_set_deviceorientation_callback_on_thread': ['malloc'],
+  'emscripten_set_focus_callback_on_thread': ['malloc'],
+  'emscripten_set_focusin_callback_on_thread': ['malloc'],
+  'emscripten_set_focusout_callback_on_thread': ['malloc'],
+  'emscripten_set_fullscreenchange_callback_on_thread': ['malloc'],
+  'emscripten_set_gamepadconnected_callback_on_thread': ['malloc'],
+  'emscripten_set_gamepaddisconnected_callback_on_thread': ['malloc'],
+  'emscripten_set_keydown_callback_on_thread': ['malloc'],
+  'emscripten_set_keypress_callback_on_thread': ['malloc'],
+  'emscripten_set_keyup_callback_on_thread': ['malloc'],
+  'emscripten_set_mousedown_callback_on_thread': ['malloc'],
+  'emscripten_set_mouseenter_callback_on_thread': ['malloc'],
+  'emscripten_set_mouseleave_callback_on_thread': ['malloc'],
+  'emscripten_set_mousemove_callback_on_thread': ['malloc'],
+  'emscripten_set_mouseout_callback_on_thread': ['malloc'],
+  'emscripten_set_mouseover_callback_on_thread': ['malloc'],
+  'emscripten_set_mouseup_callback_on_thread': ['malloc'],
+  'emscripten_set_orientationchange_callback_on_thread': ['malloc'],
+  'emscripten_set_pointerlockchange_callback_on_thread': ['malloc'],
+  'emscripten_set_resize_callback_on_thread': ['malloc'],
+  'emscripten_set_scroll_callback_on_thread': ['malloc'],
+  'emscripten_set_touchcancel_callback_on_thread': ['malloc'],
+  'emscripten_set_touchend_callback_on_thread': ['malloc'],
+  'emscripten_set_touchmove_callback_on_thread': ['malloc'],
+  'emscripten_set_touchstart_callback_on_thread': ['malloc'],
+  'emscripten_set_visibilitychange_callback_on_thread': ['malloc'],
+  'emscripten_set_wheel_callback_on_thread': ['malloc'],
   'emscripten_webgl_create_context': ['malloc'],
-  'emscripten_webgl_destroy_context': ['emscripten_webgl_make_context_current', 'emscripten_webgl_get_current_context'],
   'emscripten_webgl_get_parameter_utf8': ['malloc'],
   'emscripten_webgl_get_program_info_log_utf8': ['malloc'],
   'emscripten_webgl_get_shader_info_log_utf8': ['malloc'],
   'emscripten_webgl_get_shader_source_utf8': ['malloc'],
   'emscripten_webgl_get_supported_extensions': ['malloc'],
-  'emscripten_websocket_set_onclose_callback_on_thread': ['malloc', 'free'],
-  'emscripten_websocket_set_onerror_callback_on_thread': ['malloc', 'free'],
+  'emscripten_websocket_set_onclose_callback_on_thread': ['malloc'],
+  'emscripten_websocket_set_onerror_callback_on_thread': ['malloc'],
   'emscripten_websocket_set_onmessage_callback_on_thread': ['malloc', 'free'],
-  'emscripten_websocket_set_onopen_callback_on_thread': ['malloc', 'free'],
+  'emscripten_websocket_set_onopen_callback_on_thread': ['malloc'],
   'emscripten_wget_data': ['malloc', 'free'],
-  'gai_strerror': ['malloc'],
   'getaddrinfo': ['malloc', 'htonl', 'htons', 'ntohs'],
   'gethostbyaddr': ['malloc', 'htons'],
   'gethostbyname': ['malloc', 'htons'],
@@ -168,14 +164,9 @@ _deps_info = {
   'glfwInit': ['malloc', 'free'],
   'glfwSleep': ['sleep'],
   'glfwGetMonitors': ['malloc'],
-  'gmtime':  ['malloc'],
-  'gmtime_r':  ['malloc'],
-  'localtime': ['_get_tzname', '_get_daylight', '_get_timezone', 'malloc'],
-  'localtime_r': ['_get_tzname', '_get_daylight', '_get_timezone', 'malloc'],
-  'mktime': ['_get_tzname', '_get_daylight', '_get_timezone', 'malloc'],
-  'mmap': ['memalign'],
-  'munmap': ['free'],
-  'pthread_create': ['malloc', 'free', 'emscripten_main_thread_process_queued_calls'],
+  'localtime': ['malloc'],
+  'localtime_r': ['malloc'],
+  'mktime': ['malloc'],
   'recv': ['htons'],
   'recvmsg': ['htons'],
   'accept': ['htons'],
@@ -192,8 +183,8 @@ _deps_info = {
   'setprotoent': ['malloc'],
   'syslog': ['malloc', 'ntohs', 'htons'],
   'vsyslog': ['malloc', 'ntohs', 'htons'],
-  'timegm': ['_get_tzname', '_get_daylight', '_get_timezone', 'malloc'],
-  'tzset': ['_get_tzname', '_get_daylight', '_get_timezone', 'malloc'],
+  'timegm': ['malloc'],
+  'tzset': ['malloc'],
   'uuid_compare': ['memcmp'],
   'uuid_copy': ['memcpy'],
   'wgpuBufferGetMappedRange': ['malloc', 'free'],
@@ -205,6 +196,7 @@ _deps_info = {
 def get_deps_info():
   if not settings.EXCEPTION_HANDLING and settings.LINK_AS_CXX:
     _deps_info['__cxa_begin_catch'] = ['__cxa_is_pointer_type']
+    _deps_info['__cxa_throw'] = ['__cxa_is_pointer_type']
     _deps_info['__cxa_find_matching_catch'] = ['__cxa_can_catch']
     _deps_info['__cxa_find_matching_catch_1'] = ['__cxa_can_catch']
     _deps_info['__cxa_find_matching_catch_2'] = ['__cxa_can_catch']
@@ -215,7 +207,15 @@ def get_deps_info():
     _deps_info['__cxa_find_matching_catch_7'] = ['__cxa_can_catch']
     _deps_info['__cxa_find_matching_catch_8'] = ['__cxa_can_catch']
     _deps_info['__cxa_find_matching_catch_9'] = ['__cxa_can_catch']
+  if settings.FILESYSTEM and settings.SYSCALLS_REQUIRE_FILESYSTEM:
+    _deps_info['mmap'] = ['emscripten_builtin_memalign']
+  if settings.USE_PTHREADS and settings.OFFSCREEN_FRAMEBUFFER:
+    # When OFFSCREEN_FRAMEBUFFER is defined these functions are defined in native code,
+    # otherwise they are defined in src/library_html5_webgl.js.
+    _deps_info['emscripten_webgl_destroy_context'] = ['emscripten_webgl_make_context_current', 'emscripten_webgl_get_current_context']
+  if settings.USE_PTHREADS and settings.OFFSCREENCANVAS_SUPPORT:
+    _deps_info['emscripten_set_offscreencanvas_size_on_target_thread'] = ['emscripten_dispatch_to_thread_', 'malloc', 'free']
+    _deps_info['emscripten_set_offscreencanvas_size_on_target_thread_js'] = ['malloc']
   if settings.USE_PTHREADS:
-    _deps_info['emscripten_set_canvas_element_size_calling_thread'] = ['_emscripten_call_on_thread']
-    _deps_info['emscripten_set_offscreencanvas_size_on_target_thread'] = ['_emscripten_call_on_thread', 'malloc', 'free']
+    _deps_info['emscripten_set_canvas_element_size_calling_thread'] = ['emscripten_dispatch_to_thread_']
   return _deps_info
