@@ -11,7 +11,7 @@ from .utils import path_from_root, exit_with_error
 from . import diagnostics
 
 # Subset of settings that take a memory size (i.e. 1Gb, 64kb etc)
-MEM_SIZE_SETTINGS = (
+MEM_SIZE_SETTINGS = {
     'TOTAL_STACK',
     'INITIAL_MEMORY',
     'MEMORY_GROWTH_LINEAR_STEP',
@@ -19,9 +19,9 @@ MEM_SIZE_SETTINGS = (
     'GL_MAX_TEMP_BUFFER_SIZE',
     'MAXIMUM_MEMORY',
     'DEFAULT_PTHREAD_STACK_SIZE'
-)
+}
 
-PORTS_SETTINGS = (
+PORTS_SETTINGS = {
     # All port-related settings are valid at compile time
     'USE_SDL',
     'USE_LIBPNG',
@@ -47,11 +47,11 @@ PORTS_SETTINGS = (
     'USE_FREETYPE',
     'SDL2_MIXER_FORMATS',
     'SDL2_IMAGE_FORMATS',
-)
+}
 
 # Subset of settings that apply at compile time.
 # (Keep in sync with [compile] comments in settings.js)
-COMPILE_TIME_SETTINGS = (
+COMPILE_TIME_SETTINGS = {
     'MEMORY64',
     'INLINING_LIMIT',
     'DISABLE_EXCEPTION_CATCHING',
@@ -62,9 +62,11 @@ COMPILE_TIME_SETTINGS = (
     'STRICT',
     'EMSCRIPTEN_TRACING',
     'USE_PTHREADS',
+    'SHARED_MEMORY',
     'SUPPORT_LONGJMP',
     'DEFAULT_TO_CXX',
     'WASM_OBJECT_FILES',
+    'WASM_WORKERS',
 
     # Internal settings used during compilation
     'EXCEPTION_CATCHING_ALLOWED',
@@ -78,12 +80,13 @@ COMPILE_TIME_SETTINGS = (
     # TODO: should not be here
     'AUTO_ARCHIVE_INDEXES',
     'DEFAULT_LIBRARY_FUNCS_TO_INCLUDE',
-) + PORTS_SETTINGS
+}.union(PORTS_SETTINGS)
 
 
 class SettingsManager:
   attrs = {}
-  allowed_settings = []
+  types = {}
+  allowed_settings = set()
   legacy_settings = {}
   alt_names = {}
   internal_settings = set()
@@ -96,19 +99,24 @@ class SettingsManager:
     self.allowed_settings.clear()
 
     # Load the JS defaults into python.
-    with open(path_from_root('src', 'settings.js')) as fh:
-      settings = fh.read().replace('//', '#')
-    settings = re.sub(r'var ([\w\d]+)', r'attrs["\1"]', settings)
-    # Variable TARGET_NOT_SUPPORTED is referenced by value settings.js (also beyond declaring it),
-    # so must pass it there explicitly.
-    exec(settings, {'attrs': self.attrs})
+    def read_js_settings(filename, attrs):
+      with open(filename) as fh:
+        settings = fh.read()
+      # Use a bunch of regexs to convert the file from JS to python
+      # TODO(sbc): This is kind hacky and we should probably covert
+      # this file in format that python can read directly (since we
+      # no longer read this file from JS at all).
+      settings = settings.replace('//', '#')
+      settings = re.sub(r'var ([\w\d]+)', r'attrs["\1"]', settings)
+      settings = re.sub(r'=\s+false\s*;', '= False', settings)
+      settings = re.sub(r'=\s+true\s*;', '= True', settings)
+      exec(settings, {'attrs': attrs})
 
-    with open(path_from_root('src', 'settings_internal.js')) as fh:
-      settings = fh.read().replace('//', '#')
-    settings = re.sub(r'var ([\w\d]+)', r'attrs["\1"]', settings)
     internal_attrs = {}
-    exec(settings, {'attrs': internal_attrs})
+    read_js_settings(path_from_root('src/settings.js'), self.attrs)
+    read_js_settings(path_from_root('src/settings_internal.js'), internal_attrs)
     self.attrs.update(internal_attrs)
+    self.infer_types()
 
     if 'EMCC_STRICT' in os.environ:
       self.attrs['STRICT'] = int(os.environ.get('EMCC_STRICT'))
@@ -132,6 +140,10 @@ class SettingsManager:
 
     self.internal_settings.update(internal_attrs.keys())
 
+  def infer_types(self):
+    for key, value in self.attrs.items():
+      self.types[key] = type(value)
+
   def dict(self):
     return self.attrs
 
@@ -141,7 +153,7 @@ class SettingsManager:
   def limit_settings(self, allowed):
     self.allowed_settings.clear()
     if allowed:
-      self.allowed_settings.extend(allowed)
+      self.allowed_settings.update(allowed)
 
   def __getattr__(self, attr):
     if self.allowed_settings:
@@ -176,17 +188,34 @@ class SettingsManager:
 
     if name not in self.attrs:
       msg = "Attempt to set a non-existent setting: '%s'\n" % name
-      suggestions = difflib.get_close_matches(name, list(self.attrs.keys()))
+      valid_keys = set(self.attrs.keys()).difference(self.internal_settings)
+      suggestions = difflib.get_close_matches(name, valid_keys)
       suggestions = [s for s in suggestions if s not in self.legacy_settings]
       suggestions = ', '.join(suggestions)
       if suggestions:
         msg += ' - did you mean one of %s?\n' % suggestions
-      msg += " - perhaps a typo in emcc's  -s X=Y  notation?\n"
+      msg += " - perhaps a typo in emcc's  -sX=Y  notation?\n"
       msg += ' - (see src/settings.js for valid values)'
       exit_with_error(msg)
 
+    self.check_type(name, value)
     self.attrs[name] = value
 
+  def check_type(self, name, value):
+    if name in ('SUPPORT_LONGJMP', 'PTHREAD_POOL_SIZE', 'SEPARATE_DWARF', 'LTO'):
+      return
+    expected_type = self.types.get(name)
+    if not expected_type:
+      return
+    # Allow itegers 1 and 0 for type `bool`
+    if expected_type == bool:
+      if value in (1, 0):
+        value = bool(value)
+      if value in ('True', 'False', 'true', 'false'):
+        exit_with_error('attempt to set `%s` to `%s`; use 1/0 to set boolean settings' % (name, value))
+    if type(value) != expected_type:
+      exit_with_error('setting `%s` expects `%s` but got `%s`' % (name, expected_type.__name__, type(value).__name__))
+
   def __getitem__(self, key):
     return self.attrs[key]
 
